# sphinx_gallery_thumbnail_number = 4
from __future__ import absolute_import
from . import _graph as __graph
from ._graph import *

from .. import Configuration
from . import opt

from . opt import multicut
from . opt import lifted_multicut
from . opt import mincut
from . opt import minstcut

import numpy
from functools import partial
import types
import sys

__all__ = []

for key in __graph.__dict__.keys():
    try:
        __graph.__dict__[key].__module__='nifty.graph'
    except:
        pass
    __all__.append(key)


UndirectedGraph.__module__ = "nifty.graph"


ilpSettings = multicut.ilpSettings


# multicut objective
UndirectedGraph.MulticutObjective                     = multicut.MulticutObjectiveUndirectedGraph
UndirectedGraph.EdgeContractionGraph                  = EdgeContractionGraphUndirectedGraph
EdgeContractionGraphUndirectedGraph.MulticutObjective = multicut.MulticutObjectiveEdgeContractionGraphUndirectedGraph


UndirectedGraph.MincutObjective                     = mincut.MincutObjectiveUndirectedGraph
UndirectedGraph.EdgeContractionGraph                = EdgeContractionGraphUndirectedGraph
EdgeContractionGraphUndirectedGraph.MincutObjective = mincut.MincutObjectiveEdgeContractionGraphUndirectedGraph

# #minstcut objective
# UndirectedGraph.MinstcutObjective                     = minstcut.MinstcutObjectiveUndirectedGraph
# UndirectedGraph.EdgeContractionGraph                = EdgeContractionGraphUndirectedGraph
# EdgeContractionGraphUndirectedGraph.MinstcutObjective = minstcut.MinstcutObjectiveEdgeContractionGraphUndirectedGraph

# lifted multicut objective
UndirectedGraph.LiftedMulticutObjective = lifted_multicut.LiftedMulticutObjectiveUndirectedGraph


def randomGraph(numberOfNodes, numberOfEdges):
    g = UndirectedGraph(numberOfNodes)

    uv = numpy.random.randint(low=0, high=numberOfNodes-1, size=numberOfEdges*2)
    uv = uv.reshape([-1,2])

    where = numpy.where(uv[:,0]!=uv[:,1])[0]
    uv = uv[where,:]

    g.insertEdges(uv)
    while( g.numberOfEdges < numberOfEdges):
        u,v = numpy.random.randint(low=0, high=numberOfNodes-1, size=2)
        if u != v:
            g.insertEdge(int(u),int(v))
    return g


class EdgeContractionGraphCallback(EdgeContractionGraphCallbackImpl):
    def __init__(self):
        super(EdgeContractionGraphCallback, self).__init__()

        try:
            self.contractEdgeCallback = self.contractEdge
        except AttributeError:
            pass

        try:
            self.mergeEdgesCallback = self.mergeEdges
        except AttributeError:
            pass

        try:
            self.mergeNodesCallback = self.mergeNodes
        except AttributeError:
            pass

        try:
            self.contractEdgeDoneCallback = self.contractEdgeDone
        except AttributeError:
            pass


def edgeContractionGraph(g, callback):
    Ecg = g.__class__.EdgeContractionGraph
    ecg = Ecg(g, callback)
    return ecg


def undirectedGraph(numberOfNodes):
    return UndirectedGraph(numberOfNodes)


def undirectedGridGraph(shape, simpleNh=True):
    if not simpleNh:
        raise RuntimeError("currently only simpleNh is implemented")
    s = [int(s) for s in shape]
    if(len(s) == 2):
        return UndirectedGridGraph2DSimpleNh(s)
    elif(len(s) == 3):
        return UndirectedGridGraph3DSimpleNh(s)
    else:
        raise RuntimeError("currently only 2D and 3D grid graph is exposed to python")

gridGraph = undirectedGridGraph


def drawGraph(graph, method='spring'):
    import networkx

    G = networkx.Graph()
    for node in graph.nodes():
        G.add_node(node)

    for edge in graph.edges():
        u, v = graph.uv(edge)
        G.add_edge(u, v)

    nodeLabels = {node: str(node) for node in graph.nodes()}

    if method == 'spring':
        networkx.draw_spring(G, labels=nodeLabels)
    else:
        networkx.draw(G, lables=nodeLabels)


def run_label_propagation(graph, edge_values=None, nb_iter=1, local_edges=None, size_constr=-1,
                          nb_threads=-1):
    """
    This function can be useful to obtain superpixels (alternative to WS superpixels for example).

    The usual label propagation algorithm (https://en.wikipedia.org/wiki/Label_propagation_algorithm) iterates
        over nodes of the graph in a random order: for every iteration and selected node u,
        the algorithm assigns u to the label occurring with the highest frequency among its neighbours
        (if there are multiple highest frequency labels, it selects a label at random).
        This process can be repeated multiple times (`nb_iter`) until the algorithm converges to a set of labels.


    This generalized implementation also supports signed edge values, so that node labels are not assigned to the neighboring
        label with higher frequency, but to the neighboring label with the highest positive edge interaction.
        By default, all edge values have weight +1 and the standard label propagation algorithm is performed.

        For example, a node with the following five-nodes neighborhood:

         - neighbor_1_label = 1, edge_weight = +2
         - neighbor_2_label = 1, edge_weight = +5
         - neighbor_3_label = 1, edge_weight = -2
         - neighbor_4_label = 2, edge_weight = -5
         - neighbor_5_label = 3, edge_weight = +5

        will be randomly assigned to label 1 or 3 (given they have equal maximum attraction +5).

    :param graph:       undirected graph
    :param edge_values: Optional signed edge weights. By default, all edges have equal weight +1 and the standard
                        label propagation algorithm is performed .
    :param nb_iter:     How many label propagation iterations to perform
                        (one iteration = one loop over all the nodes of the graph)
    :param local_edges: Boolean array indicating which edges are local edges in the graph. If specified, then the
                        algorithm proceeds as following: any given node can be assigned to the label of
                        a neighboring cluster only if this cluster has at least one local edge connection to the node.
    :param size_constr: Whether or not to set a maximum size for the final clusters.
                        The default value is -1 and no size constraint is applied.
    :param nb_threads:  When multiple threads are used, multiple nodes are processed in parallel.

    :return: Newly assigned node labels
    """
    nb_edges = graph.numberOfEdges
    edge_values = numpy.ones((nb_edges,), dtype="float32") if edge_values is None else edge_values
    assert edge_values.shape[0] == nb_edges

    if local_edges is not None:
        assert edge_values.shape == local_edges.shape
        local_edges = numpy.require(local_edges, dtype='bool')
    else:
        local_edges = numpy.ones_like(edge_values).astype('bool')

    # TODO: add support initial node_labels (need to specify initial cluster size)
    nb_nodes = graph.numberOfNodes
    node_labels = numpy.arange(0, nb_nodes)
    # if node_labels is None:
    #     node_labels = numpy.arange(0, nb_nodes)
    #     sizes = numpy.ones((nb_nodes,))
    # else:
    #     raise NotImplementedError()
    node_labels = numpy.require(node_labels, dtype='uint64')

    runLabelPropagation_impl(graph, node_labels, edge_values, local_edges, nb_iter, size_constr, nb_threads)

    return node_labels


import numpy as np
import nifty.graph.rag as nrag

def accumulate_affinities_mean_and_length(affinities, offsets, labels, graph=None,
                                          affinities_weights=None,
                                          offset_weights=None,
                                          ignore_label=None, number_of_threads=-1):
    """
    Features of this function (additional ones compared to other accumulate functions):
      - does not require a RAG but simply a graph and a label image (can include long-range edges)
      - can perform weighted average of affinities depending on given affinitiesWeights
      - ignore pixels with ignore label

    Parameters
    ----------
    affinities: offset channels expected to be the first one
    """
    affinities = np.require(affinities, dtype='float32')

    if affinities_weights is not None:
        assert offset_weights is None, "Affinities weights and offset weights cannot be passed at the same time"
        affinities_weights = np.require(affinities_weights, dtype='float32')

    else:
        affinities_weights = np.ones_like(affinities)
        if offset_weights is not None:
            offset_weights = np.require(offset_weights, dtype='float32')
            for _ in range(affinities_weights.ndim-1):
                offset_weights = np.expand_dims(offset_weights, axis=-1)
            affinities_weights *= offset_weights

    affinities = np.rollaxis(affinities, axis=0, start=len(affinities.shape))
    affinities_weights = np.rollaxis(affinities_weights, axis=0, start=len(affinities_weights.shape))

    offsets = np.require(offsets, dtype='int32')
    assert len(offsets.shape) == 2

    if graph is None:
        graph = nrag.gridRag(labels)


    hasIgnoreLabel = (ignore_label is not None)
    ignore_label = 0 if ignore_label is None else int(ignore_label)

    number_of_threads = -1 if number_of_threads is None else number_of_threads

    edge_indicators_mean, edge_indicators_max, edge_sizes = \
        accumulateAffinitiesMeanAndLength_impl_(
            graph,
            labels.astype('uint64'),
            affinities,
            affinities_weights,
            offsets,
            hasIgnoreLabel,
            ignore_label,
            number_of_threads
        )
    return edge_indicators_mean, edge_sizes


def accumulate_affinities_mean_and_length_inside_clusters(affinities, offsets, labels,
                                                          offset_weights=None,
                                                          ignore_label=None, number_of_threads=-1):
    """
    Similar idea to `accumulate_affinities_mean_and_length`, but accumulates affinities/edge-values for all edges not on
     cut (i.e. connecting nodes in the same cluster)
    """
    affinities = np.require(affinities, dtype='float32')
    affinities = np.rollaxis(affinities, axis=0, start=len(affinities.shape))

    offsets = np.require(offsets, dtype='int32')
    assert len(offsets.shape) == 2

    if offset_weights is None:
        offset_weights = np.ones(offsets.shape[0], dtype='float32')
    else:
        offset_weights = np.require(offset_weights, dtype='float32')

    hasIgnoreLabel = (ignore_label is not None)
    ignore_label = 0 if ignore_label is None else int(ignore_label)

    number_of_threads = -1 if number_of_threads is None else number_of_threads

    edge_indicators_mean, edge_indicators_max, edge_sizes = \
        accumulateAffinitiesMeanAndLengthInsideClusters_impl_(
            labels.astype('uint64'),
            labels.max(),
            affinities,
            offsets,
            offset_weights,
            hasIgnoreLabel,
            ignore_label,
            number_of_threads
        )
    return edge_indicators_mean, edge_sizes


from . import _skeletons as __skeletons
from ._skeletons import *

__all__ = []
for key in __skeletons.__dict__.keys():
    try:
        __skeletons.__dict__[key].__module__='nifty.skeletons'
    except:
        pass
    __all__.append(key)



from __future__ import print_function
import nifty
import nifty
import numpy
import unittest


class TestUndirectedGridGraph(unittest.TestCase):

    def test2DSimpleNh(self):

        shape = [3,4]
        g =  nifty.graph.undirectedGridGraph(shape)

        self.assertEqual(g.numberOfNodes,12)

        def vi(x0, x1):
            return x0*shape[1] + x1

        counter = 0
        eCounter = 0
        for x0 in range(shape[0]):
            for x1 in range(shape[1]):

                u = vi(x0, x1)
                self.assertEqual(u, counter)
                coord = g.nodeToCoordinate(u)

                self.assertEqual(g.coordinateToNode(coord),u)
                self.assertEqual(g.coordinateToNode(coord),counter)
                self.assertEqual(g.coordinateToNode([x0,x1]),counter)


                self.assertEqual(coord[0], x0)
                self.assertEqual(coord[1], x1)

                if x0 + 1 < shape[0]:
                    v = vi(x0 + 1, x1)
                    e = g.findEdge(u,v)
                    uu,vv = g.uv(e)
                    self.assertNotEqual(uu,vv)
                    self.assertIn(uu,[u,v])
                    self.assertIn(vv,[u,v])
                    self.assertGreater(e,-1)
                    eCounter += 1
                if x1 + 1 < shape[1]:
                    v = vi(x0, x1 + 1)
                    e = g.findEdge(u,v)
                    uu,vv = g.uv(e)
                    self.assertNotEqual(uu,vv)
                    self.assertIn(uu,[u,v])
                    self.assertIn(vv,[u,v])
                    self.assertGreater(e,-1)
                    eCounter += 1
                counter += 1

        self.assertEqual(eCounter,g.numberOfEdges)
        for node in g.nodes():
            coord = g.nodeToCoordinate(node)
            self.assertEqual(g.coordinateToNode(coord),node)

        for edge in g.nodes():
            uv = g.uv(edge)


if __name__ == '__main__':
    unittest.main()



from __future__ import print_function

import unittest
import numpy

import nifty
import nifty.graph
import nifty.graph.agglo
nagglo = nifty.graph.agglo


class TestAgglo(unittest.TestCase):
    def testUndirectedGraph(self):
        g = nifty.graph.UndirectedGraph(4)
        edges = numpy.array([[0, 1], [0, 2], [0, 3]], dtype='uint64')
        g.insertEdges(edges)

        edgeIndicators = numpy.ones(g.numberOfEdges)
        edgeSizes = numpy.ones(g.numberOfEdges)
        nodeSizes = numpy.ones(g.numberOfNodes)

        clusterPolicy = nagglo.edgeWeightedClusterPolicy(
            graph=g, edgeIndicators=edgeIndicators,
            edgeSizes=edgeSizes, nodeSizes=nodeSizes)

        agglomerativeClustering = nagglo.agglomerativeClustering(clusterPolicy)
        agglomerativeClustering.run()
        # TODO actually test something
        seg = agglomerativeClustering.result()


if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import

from .. import Configuration

import numpy

from . import _rag as __rag
from ._rag import *

__all__ = []
for key in __rag.__dict__.keys():
    try:
        __rag.__dict__[key].__module__ = 'nifty.graph.rag'
    except:
        pass

    __all__.append(key)

try:
    import h5py
    WITH_H5PY = True
except ImportError:
    WITH_H5PY = False


def gridRag(labels,
            numberOfLabels=None,
            blockShape=None,
            numberOfThreads=-1,
            serialization=None,
            dtype='uint32'):
    labels = numpy.require(labels, dtype=dtype)
    dim = labels.ndim
    numberOfLabels = labels.max() + 1 if numberOfLabels is None else numberOfLabels
    blockShape_ = [100] * dim if blockShape is None else blockShape

    if dim == 2:
        if serialization is None:
            return explicitLabelsGridRag2D(labels,
                                           blockShape=blockShape_,
                                           numberOfLabels=numberOfLabels,
                                           numberOfThreads=int(numberOfThreads))
        else:
            return explicitLabelsGridRag2D(labels,
                                           numberOfLabels=numberOfLabels,
                                           serialization=serialization)

    elif dim == 3:
        factory = explicitLabelsGridRag3D32 if dtype == numpy.dtype('uint32') \
            else explicitLabelsGridRag3D64
        if serialization is None:
            return factory(labels,
                           blockShape=blockShape_,
                           numberOfLabels=numberOfLabels,
                           numberOfThreads=int(numberOfThreads))
        else:
            return factory(labels,
                           numberOfLabels=numberOfLabels,
                           serialization=serialization)

    else:
        raise RuntimeError("wrong dimension, currently only 2D and 3D is implemented")


def gridRagStacked2D(labels,
                     numberOfLabels,
                     serialization=None,
                     numberOfThreads=-1,
                     ignoreLabel=-1,
                     dtype='uint32'):

    labels = numpy.require(labels, dtype=dtype)
    assert labels.ndim == 3, "Stacked rag is only available for 3D labels"
    factory = gridRagStacked2D32 if dtype == numpy.dtype('uint32') \
        else gridRagStacked2D64
    if serialization is None:
        return factory(labels,
                       numberOfLabels=numberOfLabels,
                       ignoreLabel=ignoreLabel,
                       numberOfThreads=numberOfThreads)
    else:
        return factory(labels,
                       numberOfLabels=numberOfLabels,
                       ignoreLabel=ignoreLabel,
                       serialization=serialization)


# helper class for rag coordinates
def ragCoordinates(rag, numberOfThreads=-1):
    if len(rag.shape) == 2:
        return coordinatesFactoryExplicit2d(rag, numberOfThreads=numberOfThreads)
    else:
        return coordinatesFactoryExplicit3d(rag, numberOfThreads=numberOfThreads)


def ragCoordinatesStacked(rag, numberOfThreads=-1):
    return coordinatesFactoryStackedRag3d(rag, numberOfThreads=numberOfThreads)


if Configuration.WITH_HDF5:

    def gridRagHdf5(labels,
                    numberOfLabels,
                    blockShape=None,
                    numberOfThreads=-1,
                    dtype='uint32'):

        dim = labels.ndim
        blockShape_ = [100] * dim if blockShape is None else blockShape

        if dim == 2:
            return gridRag2DHdf5(labels,
                                 numberOfLabels=numberOfLabels,
                                 blockShape=blockShape_,
                                 numberOfThreads=int(numberOfThreads))
        elif dim == 3:
            factory = gridRag3DHdf532 if dtype == numpy.dtype('uint32') \
                else gridRag3DHdf564
            return factory(labels,
                           numberOfLabels=numberOfLabels,
                           blockShape=blockShape_,
                           numberOfThreads=int(numberOfThreads))
        else:
            raise RuntimeError("gridRagHdf5 is only implemented for 2D and 3D not for %dD" % dim)


    def gridRagStacked2DHdf5(labels,
                             numberOfLabels,
                             numberOfThreads=-1,
                             ignoreLabel=-1,
                             serialization=None,
                             dtype='uint32'):
        assert labels.ndim == 3, "Stacked rag is only available for 3D labels"
        factory = gridRagStacked2DHdf532 if dtype == numpy.dtype('uint32') \
            else gridRagStacked2DHdf564
        if serialization is None:
            return factory(labels,
                           numberOfLabels=numberOfLabels,
                           ignoreLabel=ignoreLabel,
                           numberOfThreads=int(numberOfThreads))
        else:
            return factory(labels,
                           ignoreLabel=ignoreLabel,
                           numberOfLabels=numberOfLabels,
                           serialization=serialization)


if Configuration.WITH_Z5:

    def gridRagZ5(labels,
                  numberOfLabels,
                  blockShape=None,
                  numberOfThreads=-1,
                  serialization=None,
                  dtype='uint32'):

        dim = len(labels.shape)
        assert dim == 3
        blockShape_ = [100] * dim if blockShape is None else blockShape

        if dim == 2:
            return gridRag2DZ5(labels,
                               numberOfLabels=numberOfLabels,
                               blockShape=blockShape_,
                               numberOfThreads=int(numberOfThreads))
        elif dim == 3:
            factory = gridRag3DZ532 if dtype == numpy.dtype('uint32') else gridRag3DZ564
            if serialization is None:
                return factory(labels,
                               numberOfLabels=numberOfLabels,
                               blockShape=blockShape_,
                               numberOfThreads=int(numberOfThreads))
            else:
                return factory(labels,
                               numberOfLabels=numberOfLabels,
                               serialization=serialization)
        else:
            raise RuntimeError("gridRagZ5 is only implemented for 2D and 3D not for %dD" % dim)


    def gridRagStacked2DZ5(labels,
                           numberOfLabels,
                           numberOfThreads=-1,
                           ignoreLabel=-1,
                           serialization=None,
                           dtype='uint32'):

        dim = len(labels.shape)
        assert dim == 3, "Stacked rag is only available for 3D labels"

        factory = gridRagStacked2DZ532 if dtype == numpy.dtype('uint32') \
            else gridRagStacked2DZ564
        if serialization is None:
            return factory(labels,
                           numberOfLabels=numberOfLabels,
                           ignoreLabel=ignoreLabel,
                           numberOfThreads=int(numberOfThreads))
        else:
            return factory(labels,
                           ignoreLabel=ignoreLabel,
                           numberOfLabels=numberOfLabels,
                           serialization=serialization)


if WITH_H5PY:

    def writeStackedRagToHdf5(rag, savePath):
        with h5py.File(savePath, 'a') as f:
            f.create_dataset('numberOfNodes', data=rag.numberOfNodes)
            f.create_dataset('numberOfEdges', data=rag.numberOfEdges)
            f.create_dataset('uvIds', data=rag.uvIds())
            f.create_dataset('minMaxLabelPerSlice', data=rag.minMaxLabelPerSlice())
            f.create_dataset('numberOfNodesPerSlice', data=rag.numberOfNodesPerSlice())
            f.create_dataset('numberOfInSliceEdges', data=rag.numberOfInSliceEdges())
            f.create_dataset('numberOfInBetweenSliceEdges',
                             data=rag.numberOfInBetweenSliceEdges())
            f.create_dataset('inSliceEdgeOffset', data=rag.inSliceEdgeOffset())
            f.create_dataset('betweenSliceEdgeOffset', data=rag.betweenSliceEdgeOffset())
            f.create_dataset('totalNumberOfInSliceEdges', data=rag.totalNumberOfInSliceEdges)
            f.create_dataset('totalNumberOfInBetweenSliceEdges',
                             data=rag.totalNumberOfInBetweenSliceEdges)
            f.create_dataset('edgeLengths', data=rag.edgeLengths())

            className = rag.__class__.__name__

            if className.startswith('GridRagStacked2DZ5'):
                factory = 'gridRagStacked2DZ5'
            elif className.startswith('GridRagStacked2DHdf5'):
                factory = 'gridRagStacked2DHdf5'
            elif className.startswith('GridRagStacked2D'):
                factory = 'gridRagStacked2D'

            # write factory and dtype as attributes
            attrs = f.attrs
            attrs['factory'] = factory
            dtype = 'uint' + className[-2:]
            attrs['dtype'] = dtype
            attrs['haveIgnoreLabel'] = rag.haveIgnoreLabel
            attrs['ignoreLabel'] = rag.ignoreLabel

    def readStackedRagFromHdf5(labels, numberOfLabels, savePath):
        serialization = []
        # load the serialization from h5
        with h5py.File(savePath, 'r') as f:
            # serialization of the undirected graph
            eAndN = numpy.zeros((2, 1), dtype='uint64')
            eAndN[0] = f['numberOfNodes']
            eAndN[1] = f['numberOfEdges']
            serialization.append(eAndN)
            serialization.append(f['uvIds'][:].ravel().astype('uint64', copy=False)[:, None])

            # serialization of the stacked rag
            tEdges = numpy.zeros((2, 1), dtype='uint64')
            tEdges[0] = f['totalNumberOfInSliceEdges']
            tEdges[1] = f['totalNumberOfInBetweenSliceEdges']
            serialization.append(tEdges)

            # load all the per slice data to squeeze it in the format we need for serializing
            # cf. nifty/include/nifty/graph/rag/grid_rag_stacked_2d.hxx serialize
            inSliceDataKeys = ['numberOfInSliceEdges',
                               'numberOfInBetweenSliceEdges',
                               'inSliceEdgeOffset',
                               'betweenSliceEdgeOffset']
            perSliceData = numpy.concatenate([numpy.array(f[key], dtype='uint64')[:, None]
                                              for key in inSliceDataKeys], axis=0)
            minmaxLabel = f['minMaxLabelPerSlice'][:]
            perSliceData = numpy.concatenate([perSliceData, minmaxLabel[:, :1], minmaxLabel[:, 1:]],
                                             axis=0)
            serialization.append(perSliceData.astype('uint64', copy=False))
            serialization.append(f['edgeLengths'][:].astype('uint64', copy=False)[:, None])

            # read factory and dtype from attributes
            attrs = f.attrs
            factory = eval(attrs['factory'])
            dtype = attrs['dtype']
            ignoreLabel = attrs['ignoreLabel'] if attrs['haveIgnoreLabel'] else -1

        serialization = numpy.concatenate(serialization)

        return factory(labels,
                       dtype=dtype,
                       numberOfLabels=numberOfLabels,
                       ignoreLabel=ignoreLabel,
                       serialization=serialization.squeeze())


def compute_lifted_edges_from_rag_and_offsets(rag, offsets, numberOfThreads=-1):
    if isinstance(offsets, numpy.ndarray):
        offsets = offsets.tolist()
    else:
        assert isinstance(offsets, (list, tuple))
        assert isinstance(offsets, (list, tuple))
    return computeLiftedEdgesFromRagAndOffsets_impl(rag, offsets, numberOfThreads)




from __future__ import print_function
import unittest
import random

import numpy
import nifty
import nifty.graph


class TestLiftedMulticutSolver(unittest.TestCase):

    def generateGrid(self, gridSize):
        def nid(x, y):
            return x*gridSize[1] + y
        G = nifty.graph.UndirectedGraph
        g =  G(gridSize[0] * gridSize[1])
        for x in range(gridSize[0]):
            for y in range(gridSize[1]):

                u = nid(x,y)

                if x + 1 < gridSize[0]:

                    v = nid(x+1, y)
                    g.insertEdge(u, v)

                if y + 1 < gridSize[1]:

                    v = nid(x, y+1)
                    g.insertEdge(u, v)

        return g, nid

    def gridLiftedModel(self, gridSize=[3, 2], bfsRadius=2, weightRange=[-1, 1]):
        g,nid = self.generateGrid(gridSize)
        obj = nifty.graph.lifted_multicut.liftedMulticutObjective(g)
        graph = obj.graph
        liftedGraph = obj.liftedGraph


        # this should  add edges
        obj.insertLiftedEdgesBfs(bfsRadius)
        postEdges = liftedGraph.numberOfEdges


        for edge in liftedGraph.edges():
            u,v = liftedGraph.uv(edge)
            w = random.uniform(weightRange[0],weightRange[1])
            obj.setCost(u, v, w)

        return obj,nid

    def testLiftedMulticutGreedyAdditive(self):
        for x in range(5):
            obj,nid = self.gridLiftedModel(gridSize=[40,40] , bfsRadius=4, weightRange=[-1,1])


            solverFactory = obj.liftedMulticutGreedyAdditiveFactory()
            solver = solverFactory.create(obj)
            visitor = obj.verboseVisitor(100)
            argN = solver.optimize()

            if False:
                solverFactory = obj.liftedMulticutAndresGreedyAdditiveFactory()
                solver = solverFactory.create(obj)
                visitor = obj.verboseVisitor(100)
                argA = solver.optimize()

                self.assertAlmostEqual(obj.evalNodeLabels(argA), obj.evalNodeLabels(argN))

    def testLiftedMulticutKernighanLinSimple(self):
        G = nifty.graph.UndirectedGraph
        g = G(5);
        g.insertEdge(0, 1) # 0
        g.insertEdge(0, 3) # 1
        g.insertEdge(1, 2) # 2
        g.insertEdge(1, 4) #
        g.insertEdge(3, 4) # 4

        obj = nifty.graph.lifted_multicut.liftedMulticutObjective(g)
        lg = obj.liftedGraph

        obj.setCost(0, 1, 10.0)
        obj.setCost(0, 2, -1.0)
        obj.setCost(0, 3, -1.0)
        obj.setCost(0, 4, -1.0)
        obj.setCost(1, 2, 10.0)
        obj.setCost(1, 3, -1.0)
        obj.setCost(1, 4,  4.0)
        obj.setCost(2, 3, -1.0)
        obj.setCost(2, 4, -1.0)
        obj.setCost(3, 4, 10.0)

        arg = numpy.array([1,2,3,4,5])
        solverFactory = obj.liftedMulticutKernighanLinFactory()
        solver = solverFactory.create(obj)
        visitor = obj.verboseVisitor(100)

        arg2 = solver.optimize(visitor,arg)

        self.assertEqual(arg2[0] != arg2[1],  bool(0))
        self.assertEqual(arg2[0] != arg2[2],  bool(0))
        self.assertEqual(arg2[0] != arg2[3],  bool(1))
        self.assertEqual(arg2[0] != arg2[4],  bool(1))
        self.assertEqual(arg2[1] != arg2[2],  bool(0))
        self.assertEqual(arg2[1] != arg2[3],  bool(1))
        self.assertEqual(arg2[1] != arg2[4],  bool(1))
        self.assertEqual(arg2[2] != arg2[3],  bool(1))
        self.assertEqual(arg2[2] != arg2[4],  bool(1))
        self.assertEqual(arg2[3] != arg2[4],  bool(0))

    def testLiftedMulticutKernighanLin(self):
        gridSize = [5, 5]
        for x in range(4):
            obj,nid = self.gridLiftedModel(gridSize=gridSize , bfsRadius=2, weightRange=[-3,1])

            solverFactory = obj.liftedMulticutGreedyAdditiveFactory()
            solver = solverFactory.create(obj)
            visitor = obj.verboseVisitor(100)
            arg = solver.optimize()
            argGC = arg.copy()
            egc = obj.evalNodeLabels(argGC)

            #arg = numpy.arange(gridSize[0]*gridSize[1])
            solverFactory = obj.liftedMulticutKernighanLinFactory()
            solver = solverFactory.create(obj)
            visitor = obj.verboseVisitor(100)
            arg2 = solver.optimize(argGC.copy())
            ekl = obj.evalNodeLabels(arg2)

            if False:
                solverFactory = obj.liftedMulticutAndresKernighanLinFactory()
                solver = solverFactory.create(obj)
                visitor = obj.verboseVisitor(100)
                arg3 = solver.optimize(argGC.copy())
                eakl = obj.evalNodeLabels(arg3)

                self.assertLessEqual(ekl, egc)
                self.assertAlmostEqual(ekl, eakl)

        for x in range(4):
            obj,nid = self.gridLiftedModel(gridSize=gridSize , bfsRadius=2, weightRange=[-3,1])

            arg = numpy.arange(gridSize[0]*gridSize[1])

            solverFactory = obj.liftedMulticutKernighanLinFactory()
            solver = solverFactory.create(obj)
            visitor = obj.verboseVisitor(100)
            arg2 = solver.optimize(arg.copy())
            ekl = obj.evalNodeLabels(arg2)

            if False:
                solverFactory = obj.liftedMulticutAndresKernighanLinFactory()
                solver = solverFactory.create(obj)
                visitor = obj.verboseVisitor(100)
                arg3 = solver.optimize(arg.copy())
                eakl = obj.evalNodeLabels(arg3)

                self.assertAlmostEqual(ekl, eakl)

    def testLiftedMulticutSolverFm(self):
        random.seed(0)
        for x in range(1):
            obj, nid = self.gridLiftedModel(gridSize=[40,40] , bfsRadius=4, weightRange=[-1,1])

            pgen = obj.watershedProposalGenerator(sigma=1.0,
                                                  seedingStrategy='SEED_FROM_LOCAL',
                                                  numberOfSeeds=0.1)
            # print(x)

            solverFactory = obj.fusionMoveBasedFactory()
            solverFactory = obj.fusionMoveBasedFactory(proposalGenerator=pgen,
                                                       numberOfIterations=100,
                                                       stopIfNoImprovement=10)
            solver = solverFactory.create(obj)
            visitor = obj.verboseVisitor(100)
            argN = solver.optimize(visitor)

    def implTestLiftedMulticutIlpBfsGrid(self, ilpSolver,
                                         gridSize=[4,4], bfsRadius=4,
                                         weightRange=[-2,1], verbose=0,
                                         relativeGap=0.00001):

        obj,nid = self.gridLiftedModel(gridSize=gridSize,
                                       bfsRadius=bfsRadius,
                                       weightRange=weightRange)
        solverFactory = obj.liftedMulticutIlpFactory(ilpSolver=ilpSolver,
                                                     relativeGap=relativeGap)
        solver = solverFactory.create(obj)
        if verbose > 0:
            visitor = obj.verboseVisitor(100)
            arg = solver.optimize(visitor=visitor)
        else:
            arg = solver.optimize()

    def testLiftedMulticutIlpBfsGrid(self):
        if nifty.Configuration.WITH_GLPK:
            for x in range(5):
                self.implTestLiftedMulticutIlpBfsGrid(ilpSolver='glpk', gridSize=[3,3],
                                                      relativeGap=0.3,
                                                      weightRange=(-2,1))
        if nifty.Configuration.WITH_CPLEX:
            for x in range(5):
                self.implTestLiftedMulticutIlpBfsGrid(ilpSolver='cplex', gridSize=[3,3],
                                                      relativeGap=0.3,
                                                      verbose=0,  weightRange=(-2,1))
        if nifty.Configuration.WITH_GUROBI:
            for x in range(5):
                self.implTestLiftedMulticutIlpBfsGrid(ilpSolver='gurobi')

    def implTestSimpleChainModelOpt(self, size, ilpSolver):
        if nifty.Configuration.WITH_CPLEX:
            #  0 - 1 - 2 - 3
            #
            G = nifty.graph.UndirectedGraph
            g =  G(size)
            for x in range(size - 1):
                g.insertEdge(x, x+1)

            obj = nifty.graph.lifted_multicut.liftedMulticutObjective(g)
            graph = obj.graph
            liftedGraph = obj.liftedGraph

            for x in range(size - 1):
                w = 1.0
                obj.setCost(x, x+1, w)

            w = -1.0 * float(size)
            obj.setCost(0, size-1, w)

            solverFactory = obj.liftedMulticutIlpFactory(ilpSolver=ilpSolver)
            solver = solverFactory.create(obj)
            visitor = obj.verboseVisitor(100)
            arg = solver.optimize()

            self.assertNotEqual(arg[0],arg[size-1])

    def testSimpleChainModelMulticutGlpk(self):
        if False and nifty.Configuration.WITH_GLPK:
            for x in range(10):
                self.implTestSimpleChainModelOpt(ilpSolver='glpk', size=4)

    def testSimpleChainModelMulticutGurobi(self):
        if nifty.Configuration.WITH_GUROBI:
            for x in range(10):
                self.implTestSimpleChainModelOpt(ilpSolver='gurobi', size=4)

    def testSimpleChainModelMulticutCplex(self):
        if nifty.Configuration.WITH_CPLEX:
            for x in range(10):
                self.implTestSimpleChainModelOpt(ilpSolver='cplex', size=4)


if __name__ == '__main__':
    unittest.main()



import opengm
import vigra
import nifty
import numpy


f = "/home/tbeier/Desktop/mc_models/knott-3d-150/gm_knott_3d_039.h5"
#f = "/home/tbeier/Desktop/mc_models/knot-3d-550/gm_knott_3d_119.h5"
#f = "/home/tbeier/Desktop/mc_models/knott-3d-450/gm_knott_3d_103.h5"
#f = "/home/tbeier/Downloads/gm_large_3.gm"
#f = "/home/tbeier/Downloads/gm_small_1.gm"
#f = "/home/tbeier/Desktop/mc_models/knott-3d-300/gm_knott_3d_072.h5"
gm = opengm.loadGm(f)





def opengmToNumpy(gm):
    nNodes = gm.numberOfVariables
    nEdges = gm.numberOfFactors


    factorSubset=opengm.FactorSubset(gm)
    weights = factorSubset.evaluate([0,1])
    weightsB = factorSubset.evaluate([0,0])
    print "wb", weightsB.sum()
    vis =  factorSubset.variableIndices()
    assert len(numpy.unique(vis.reshape(-1))) == nNodes
    #print vis.shape,weights.shape
    assert vis.max()+1 == nNodes
    return nNodes,weights,vis

nNodes,weights, uvs = opengmToNumpy(gm)
nFac = weights.shape[0]


g =  nifty.graph.UndirectedGraph(int(nNodes))
g.insertEdges(uvs)
obj = nifty.graph.multicut.multicutObjective(g, weights)


ilpFactory = obj.multicutIlpFactory(ilpSolver='cplex',
    addThreeCyclesConstraints=True,
    addOnlyViolatedThreeCyclesConstraints=True
    #memLimit= 0.01
)



greedy=obj.greedyAdditiveFactory().create(obj)
visitor = obj.multicutVerboseVisitor(10)
ret = greedy.optimize(visitor=visitor)

greedy=obj.greedyAdditiveFactory()
fmFactory = obj.fusionMoveBasedFactory(
    #fusionMove=nifty.fusionMoveSettings(mcFactory=greedy),
    fusionMove=obj.fusionMoveSettings(mcFactory=ilpFactory),
    #proposalGen=nifty.greedyAdditiveProposals(sigma=30,nodeNumStopCond=-1,weightStopCond=0.0),
    proposalGen=obj.watershedProposals(sigma=1,seedFraction=0.01),
    numberOfIterations=100,
    numberOfParallelProposals=16, # no effect if nThreads equals 0 or 1
    numberOfThreads=0,
    stopIfNoImprovement=20,
    fuseN=2,
)


s = obj.perturbAndMapSettings(mcFactory=fmFactory,noiseMagnitude=4.2,numberOfThreads=-1,
                            numberOfIterations=10, noiseType='uniform')
pAndMap = obj.perturbAndMap(obj, s)
res =  pAndMap.optimize(ret)


for x in range(30):
    print res[x]," ",




import os
import unittest
import numpy as np
import nifty.graph.long_range_adjacency as nlr


# FIXME multi-threading is broken
class TestLongRangeAdjacency(unittest.TestCase):

    def generate_toy_data(self):
        seg = np.array(
            [[[1, 1, 1],
              [1, 1, 2],
              [1, 2, 3]],
             [[4, 4, 4],
              [4, 5, 5],
              [4, 5, 5]],
             [[6, 6, 6],
              [6, 7, 7],
              [6, 7, 7]]], dtype='uint32'
        )
        return seg, 8

    def generate_toy_affinities(self):
         affs = np.ones((1, 3, 3, 3), dtype='float32')
         affs[:, -1] = 0
         return affs

    def generate_random_data(self):
        seg = np.zeros((100, 100, 100), dtype='uint32')
        current_label = 0
        for z in range(seg.shape[0]):
            for y in range(seg.shape[1]):
                for x in range(seg.shape[2]):
                    seg[z, y, x] = current_label
                    if np.random.rand() > .95:
                        current_label += 1
            current_label += 1
        n_labels = int(seg.max())+ 1
        return seg, n_labels

    def generate_random_affinities(self, long_range):
        return np.random.random(size=(long_range - 1, 100, 100, 100)).astype('float32')

    def checks_toy_data(self, lr):
        self.assertEqual(lr.numberOfNodes, 8)
        self.assertEqual(lr.numberOfEdges, 4)
        uvs = lr.uvIds()
        self.assertEqual(uvs[0].tolist(), [1, 6])
        self.assertEqual(uvs[1].tolist(), [1, 7])
        self.assertEqual(uvs[2].tolist(), [2, 7])
        self.assertEqual(uvs[3].tolist(), [3, 7])

    def test_toy_data(self):
        seg, n_labels = self.generate_toy_data()
        lr = nlr.longRangeAdjacency(seg, numberOfLabels=n_labels,
                                    longRange=2, numberOfThreads=1)
        self.checks_toy_data(lr)

    def test_serialization(self):
        seg, n_labels = self.generate_toy_data()
        lr = nlr.longRangeAdjacency(seg, numberOfLabels=n_labels,
                                    longRange=2, numberOfThreads=1)
        serialization = lr.serialize()
        lr_ = nlr.longRangeAdjacency(seg, serialization=serialization)
        self.checks_toy_data(lr_)

    def test_random_data(self):
        seg, n_labels = self.generate_random_data()
        lr = nlr.longRangeAdjacency(seg, numberOfLabels=n_labels,
                                    longRange=2, numberOfThreads=1)
        self.assertEqual(lr.numberOfNodes, n_labels)
        self.assertGreater(lr.numberOfEdges, 100)

    def test_features_toy_data(self):
        seg, n_labels = self.generate_toy_data()
        affs = self.generate_toy_affinities()
        lr = nlr.longRangeAdjacency(seg, numberOfLabels=n_labels,
                                    longRange=2, numberOfThreads=1)
        expected_mean = {1: 1., 2: 0.}
        for z_dir in (1, 2):
            features = nlr.longRangeFeatures(lr, seg, affs, z_dir)
            self.assertTrue((features[:, 0] == expected_mean[z_dir]).all())
            self.assertTrue((features[:, 1] == 0.).all())

    def test_features_random_data(self):
        seg, n_labels = self.generate_random_data()
        for long_range in (2, 3, 4):
            affinities = self.generate_random_affinities(long_range)
            lr = nlr.longRangeAdjacency(seg, numberOfLabels=n_labels,
                                        longRange=long_range, numberOfThreads=1)
            for z_dir in (1, 2):
                features = nlr.longRangeFeatures(lr, seg, affinities, z_dir)
                self.assertEqual(features.shape[0], lr.numberOfEdges)
                self.assertEqual(features.shape[1], 9)
                for col in range(features.shape[1]):
                    self.assertFalse(np.sum(features[:, col]) == 0)


if __name__ == '__main__':
    unittest.main()



from . import _distributed as __distributed
from ._distributed import *

__all__ = []
for key in __distributed.__dict__.keys():
    try:
        __distributed.__dict__[key].__module__='nifty.distributed'
    except:
        pass
    __all__.append(key)



import unittest
import os
from shutil import rmtree

import numpy as np
import nifty
import nifty.graph.rag as nrag

ON_WIN = os.name == 'nt'


class TestAccumulateStacked(unittest.TestCase):
    shape = (10, 256, 256)
    # shape = (3, 128, 128)

    @staticmethod
    def make_labels(shape):
        labels = np.zeros(shape, dtype='uint32')
        label = 0
        for z in range(shape[0]):
            for y in range(shape[1]):
                for x in range(shape[2]):
                    labels[z, y, x] = label
                    if np.random.random() > .95:
                        have_increased = True
                        label += 1
                    else:
                        have_increased = False
            if not have_increased:
                label += 1
        return labels

    def setUp(self):
        self.data = np.random.random(size=self.shape).astype('float32')
        self.labels = self.make_labels(self.shape)
        self.n_labels = self.labels.max() + 1
        self.tmp_dir = './tmp'
        if not os.path.exists(self.tmp_dir):
            os.mkdir(self.tmp_dir)

    def tearDown(self):
        if os.path.exists(self.tmp_dir):
            rmtree(self.tmp_dir)

    def check_features(self, features, expected_length):
        self.assertEqual(len(features), expected_length)
        for feat_id in range(features.shape[1]):
            self.assertFalse(np.allclose(features[:, feat_id], 0.))

    def accumulation_in_core_test(self, accumulation_function):
        rag = nrag.gridRagStacked2D(self.labels,
                                    numberOfLabels=self.n_labels,
                                    numberOfThreads=-1)
        n_edges_xy = rag.totalNumberOfInSliceEdges
        n_edges_z  = rag.totalNumberOfInBetweenSliceEdges

        # test complete accumulation
        # print("Complete Accumulation ...")
        feats_xy, feats_z = accumulation_function(rag,
                                                  self.data,
                                                  numberOfThreads=-1)
        self.check_features(feats_xy, n_edges_xy)
        self.check_features(feats_z, n_edges_z)
        # print("... passed")

        # test xy-feature accumulation
        # print("Complete XY Accumulation ...")
        feats_xy, feats_z = accumulation_function(rag,
                                                  self.data,
                                                  keepXYOnly=True,
                                                  numberOfThreads=-1)
        self.check_features(feats_xy, n_edges_xy)
        self.assertEqual(len(feats_z), 1)
        # print("... passed")

        # test z-feature accumulation for all 3 directions
        # print("Complete Z Accumulations ...")
        for z_direction in (0, 1, 2):
            feats_xy, feats_z = accumulation_function(rag,
                                                      self.data,
                                                      keepZOnly=True,
                                                      zDirection=z_direction,
                                                      numberOfThreads=-1)
            self.assertEqual(len(feats_xy), 1)
            self.check_features(feats_z, n_edges_z)
        # print("... passed")

    def test_standard_features_in_core(self):
        self.accumulation_in_core_test(nrag.accumulateEdgeStandardFeatures)

    @unittest.skipUnless(nifty.Configuration.WITH_FASTFILTERS, "skipping fastfilter tests")
    def test_features_from_filters_in_core(self):
        self.accumulation_in_core_test(nrag.accumulateEdgeFeaturesFromFilters)

    def accumulation_z5_test(self, accumulation_function, n_feats):
        import z5py
        import nifty.z5
        label_path = os.path.join(self.tmp_dir, 'labels.n5')
        f_labels = z5py.File(label_path, use_zarr_format=False)
        dsl = f_labels.create_dataset('data',
                                      dtype='uint32',
                                      shape=self.shape,
                                      chunks=(1, 25, 25),
                                      compression='raw')
        dsl[:] = self.labels

        rag = nrag.gridRagStacked2DZ5(nifty.z5.datasetWrapper('uint32', label_path, 'data'),
                                      numberOfLabels=self.n_labels,
                                      numberOfThreads=1)
        n_edges_xy = rag.totalNumberOfInSliceEdges
        n_edges_z  = rag.totalNumberOfInBetweenSliceEdges

        data_path = os.path.join(self.tmp_dir, 'data.n5')
        f_data = z5py.File(data_path, use_zarr_format=False)
        dsd = f_data.create_dataset('data',
                                    dtype='float32',
                                    shape=self.shape,
                                    chunks=(1, 25, 25),
                                    compression='raw')
        dsd[:] = self.data

        def open_features(keep_xy=False, keep_z=False):
            p_xy = os.path.join(self.tmp_dir, 'xy.n5')
            p_z = os.path.join(self.tmp_dir, 'z.n5')
            f_xy = z5py.File(p_xy, use_zarr_format=False)
            f_z = z5py.File(p_z, use_zarr_format=False)
            f_xy.create_dataset('data',
                                dtype='float32',
                                shape=(1 if keep_z else n_edges_xy, n_feats),
                                chunks=(1 if keep_z else 500, n_feats),
                                compression='raw')
            f_z.create_dataset('data',
                               dtype='float32',
                               shape=(1 if keep_xy else n_edges_z, n_feats),
                               chunks=(1 if keep_xy else 500, n_feats),
                               compression='raw')
            return p_xy, p_z

        def load_features(p_xy, p_z):
            xy_feats = z5py.File(p_xy)['data'][:]
            z_feats = z5py.File(p_z)['data'][:]
            rmtree(p_xy)
            rmtree(p_z)
            return xy_feats, z_feats

        path_xy, path_z = open_features()

        # test complete accumulation
        print("Complete Accumulation ...")
        accumulation_function(rag,
                              nifty.z5.datasetWrapper('float32', data_path, 'data'),
                              nifty.z5.datasetWrapper('float32', path_xy, 'data'),
                              nifty.z5.datasetWrapper('float32', path_z, 'data'),
                              numberOfThreads=1)

        feats_xy, feats_z = load_features(path_xy, path_z)
        self.check_features(feats_xy, n_edges_xy)
        self.check_features(feats_z, n_edges_z)
        print("... passed")

        # test xy-feature accumulation
        print("Complete XY Accumulation ...")
        path_xy, path_z = open_features(keep_xy=True)
        accumulation_function(rag,
                              nifty.z5.datasetWrapper('float32', data_path, 'data'),
                              nifty.z5.datasetWrapper('float32', path_xy, 'data'),
                              nifty.z5.datasetWrapper('float32', path_z, 'data'),
                              keepXYOnly=True,
                              numberOfThreads=-1)

        feats_xy, feats_z = load_features(path_xy, path_z)

        self.check_features(feats_xy, n_edges_xy)
        self.assertEqual(len(feats_z), 1)
        print("... passed")

        # test z-feature accumulation for all 3 directions
        print("Complete Z Accumulations ...")
        for z_direction in (0, 1, 2):
            path_xy, path_z = open_features(keep_z=True)
            accumulation_function(rag,
                                  nifty.z5.datasetWrapper('float32', data_path, 'data'),
                                  nifty.z5.datasetWrapper('float32', path_xy, 'data'),
                                  nifty.z5.datasetWrapper('float32', path_z, 'data'),
                                  keepZOnly=True,
                                  numberOfThreads=-1)

            feats_xy, feats_z = load_features(path_xy, path_z)
            self.assertEqual(len(feats_xy), 1)
            self.check_features(feats_z, n_edges_z)
        print("... passed")

    @unittest.skipIf(ON_WIN, "fails on windows")
    @unittest.skipUnless(nifty.Configuration.WITH_Z5, "skipping z5 tests")
    def test_z5_standard_features(self):
        self.accumulation_z5_test(nrag.accumulateEdgeStandardFeatures, n_feats=9)

    @unittest.skipUnless(nifty.Configuration.WITH_Z5 and nifty.Configuration.WITH_FASTFILTERS,
                         "skipping z5 fastfilter tests")
    def test_z5_features_from_filters(self):
        self.accumulation_z5_test(nrag.accumulateEdgeFeaturesFromFilters, n_feats=9 * 12)

    @unittest.skipIf(ON_WIN, "fails on windows")
    @unittest.skipUnless(nifty.Configuration.WITH_Z5, "skipping z5 tests")
    def test_in_vs_out_of_core(self):
        accumulation_function = nrag.accumulateEdgeStandardFeatures
        n_feats = 9
        import z5py
        import nifty.z5

        ###############
        # get features with out of core calculation

        label_path = os.path.join(self.tmp_dir, 'labels.n5')
        f_labels = z5py.File(label_path, use_zarr_format=False)
        dsl = f_labels.create_dataset('data',
                                      dtype='uint32',
                                      shape=self.shape,
                                      chunks=(1, 25, 25),
                                      compression='raw')
        dsl[:] = self.labels

        rag_ooc = nrag.gridRagStacked2DZ5(nifty.z5.datasetWrapper('uint32', label_path, 'data'),
                                          numberOfLabels=self.n_labels,
                                          numberOfThreads=1)
        data_path = os.path.join(self.tmp_dir, 'data.n5')
        f_data = z5py.File(data_path, use_zarr_format=False)
        dsd = f_data.create_dataset('data',
                                    dtype='float32',
                                    shape=self.shape,
                                    chunks=(1, 25, 25),
                                    compression='raw')
        dsd[:] = self.data

        n_edges_xy = rag_ooc.totalNumberOfInSliceEdges
        n_edges_z  = rag_ooc.totalNumberOfInBetweenSliceEdges

        def open_features(keep_xy=False, keep_z=False):
            p_xy = os.path.join(self.tmp_dir, 'xy.n5')
            p_z = os.path.join(self.tmp_dir, 'z.n5')
            f_xy = z5py.File(p_xy, use_zarr_format=False)
            f_z = z5py.File(p_z, use_zarr_format=False)
            f_xy.create_dataset('data',
                                dtype='float32',
                                shape=(1 if keep_z else n_edges_xy, n_feats),
                                chunks=(1 if keep_z else 500, n_feats),
                                compression='raw')
            f_z.create_dataset('data',
                               dtype='float32',
                               shape=(1 if keep_xy else n_edges_z, n_feats),
                               chunks=(1 if keep_xy else 500, n_feats),
                               compression='raw')
            return p_xy, p_z

        def load_features(p_xy, p_z):
            xy_feats = z5py.File(p_xy)['data'][:]
            z_feats = z5py.File(p_z)['data'][:]
            rmtree(p_xy)
            rmtree(p_z)
            return xy_feats, z_feats

        path_xy, path_z = open_features()

        # test complete accumulation
        accumulation_function(rag_ooc,
                              nifty.z5.datasetWrapper('float32', data_path, 'data'),
                              nifty.z5.datasetWrapper('float32', path_xy, 'data'),
                              nifty.z5.datasetWrapper('float32', path_z, 'data'),
                              numberOfThreads=-1)

        feats_xy_ooc, feats_z_ooc = load_features(path_xy, path_z)

        ###############
        # get features with in core calculation

        rag = nrag.gridRagStacked2D(self.labels,
                                    numberOfLabels=self.n_labels,
                                    numberOfThreads=1)

        # test complete accumulation
        feats_xy, feats_z = accumulation_function(rag,
                                                  self.data,
                                                  numberOfThreads=1)
        self.assertEqual(feats_xy.shape, feats_xy_ooc.shape)
        self.assertTrue(np.allclose(feats_xy, feats_xy_ooc))
        self.assertEqual(feats_z.shape, feats_z_ooc.shape)
        self.assertTrue(np.allclose(feats_z, feats_z_ooc))

    @staticmethod
    def make_labels_with_ignore(shape):
        labels = np.zeros(shape, dtype='uint32')
        mask = np.random.choice([0, 1], size=shape, p=[1. / 10., 9. / 10.]).astype('bool')
        label = 0
        for z in range(shape[0]):
            for y in range(shape[1]):
                for x in range(shape[2]):
                    if not mask[z, y, x]:
                        continue
                    labels[z, y, x] = label
                    if np.random.random() > .95:
                        have_increased = True
                        label += 1
                    else:
                        have_increased = False
            if not have_increased:
                label += 1
        return labels

    def ignore_label_test_test(self, accumulation_function):
        labels_with_ignore = self.make_labels_with_ignore(self.shape)
        rag = nrag.gridRagStacked2D(labels_with_ignore,
                                    numberOfLabels=labels_with_ignore.max() + 1,
                                    ignoreLabel=0,
                                    numberOfThreads=1)
        n_edges_xy = rag.totalNumberOfInSliceEdges
        n_edges_z  = rag.totalNumberOfInBetweenSliceEdges

        # test complete accumulation
        print("Complete Accumulation ...")
        feats_xy, feats_z = accumulation_function(rag,
                                                  self.data,
                                                  numberOfThreads=1)
        self.check_features(feats_xy, n_edges_xy)
        self.check_features(feats_z, n_edges_z)
        print("... passed")

        # test xy-feature accumulation
        print("Complete XY Accumulation ...")
        feats_xy, feats_z = accumulation_function(rag,
                                                  self.data,
                                                  keepXYOnly=True,
                                                  numberOfThreads=-1)
        self.check_features(feats_xy, n_edges_xy)
        self.assertEqual(len(feats_z), 1)
        print("... passed")

        # test z-feature accumulation for all 3 directions
        print("Complete Z Accumulations ...")
        for z_direction in (0, 1, 2):
            feats_xy, feats_z = accumulation_function(rag,
                                                      self.data,
                                                      keepZOnly=True,
                                                      zDirection=z_direction,
                                                      numberOfThreads=-1)
            self.assertEqual(len(feats_xy), 1)
            self.check_features(feats_z, n_edges_z)
        print("... passed")

    def test_standard_features_ignore(self):
        self.ignore_label_test_test(nrag.accumulateEdgeStandardFeatures)


if __name__ == '__main__':
    unittest.main()



from . import _transformation as _trafo_impl
from ._transformation import parseTransformixCoordinates


def get_path_and_key_from_dataset(dataset):
    key = dataset.name
    # z5 doesn't like leading slash
    if key.startswith('/'):
        key = key[1:]
    return dataset.file.filename, key


# TODO support pre-smoothing
def affineTransformationZ5(data, matrix, order, bounding_box,
                           fill_value=0, sigma=None):
    """
    """
    ndim = data.ndim
    start = [bb.start for bb in bounding_box]
    stop = [bb.stop for bb in bounding_box]
    if ndim != len(start) != len(stop):
        raise ValueError("Invalid dimension")
    dtype = str(data.dtype)
    func = "affineTransformationZ5%iD%s" % (ndim, dtype)
    func = getattr(_trafo_impl, func)
    path, key = get_path_and_key_from_dataset(data)
    return func(path, key, matrix, order, start, stop, fill_value)


# TODO support pre-smoothing
def affineTransformationH5(data, matrix, order, bounding_box,
                           fill_value=0, sigma=None):
    """
    """
    ndim = data.ndim
    start = [bb.start for bb in bounding_box]
    stop = [bb.stop for bb in bounding_box]
    if ndim != len(start) != len(stop):
        raise ValueError("Invalid dimension")
    dtype = str(data.dtype)
    func = "affineTransformationH5%iD%s" % (ndim, dtype)
    func = getattr(_trafo_impl, func)
    path, key = get_path_and_key_from_dataset(data)
    return func(path, key, matrix, order, start, stop, fill_value)


def affineTransformation(data, matrix, order, bounding_box,
                         fill_value=0):
    """
    """
    ndim = data.ndim
    start = [bb.start for bb in bounding_box]
    stop = [bb.stop for bb in bounding_box]
    if ndim != len(start) != len(stop):
        raise ValueError("Invalid dimension")
    dtype = str(data.dtype)
    func = "affineTransformation%iD%s" % (ndim, dtype)
    func = getattr(_trafo_impl, func)
    return func(data, matrix, order, start, stop, fill_value)


def coordinateTransformationZ5(data, coordinate_file, bounding_box, fill_value=0):
    """
    """
    ndim = data.ndim
    start = [bb.start for bb in bounding_box]
    stop = [bb.stop for bb in bounding_box]
    if ndim != len(start) != len(stop):
        raise ValueError("Invalid dimension")
    dtype = str(data.dtype)
    func = "coordinateTransformationZ5%iD%s" % (ndim, dtype)
    func = getattr(_trafo_impl, func)
    path, key = get_path_and_key_from_dataset(data)
    return func(path, key, coordinate_file, start, stop, fill_value)



import nifty.cgp as ncgp
import nifty.graph.rag as nrag
import unittest

import nifty
import unittest
import nifty.cgp as ncgp
import numpy

numpy.random.seed(42)


class TestCgp2d(unittest.TestCase):

    def test_corner_case_3x3_grid_a(self):

        assertEq = self.assertEqual

        # 4 one cells are active
        # but still  no junction
        seg = [
            [1,1,2],
            [1,3,1],
            [1,1,1]
        ]
        seg = numpy.array(seg,dtype='uint32')
        tGrid = ncgp.TopologicalGrid2D(seg)

        numberOfCells  = tGrid.numberOfCells
        assertEq(numberOfCells,[0,2,3])

        tShape = tGrid.topologicalGridShape
        assertEq(tShape, [5,5])

        shape = tGrid.shape
        assertEq(shape, [3,3])


        # check the bounds
        bounds = tGrid.extractCellsBounds()

        bounds0 = bounds[0]
        bounds1 = bounds[1]
        boundedBy1 = bounds0.reverseMapping()
        boundedBy2 = bounds1.reverseMapping()

        assertEq(len(bounds0),0)
        assertEq(len(bounds1),2)
        assertEq(len(boundedBy1),2)
        assertEq(len(boundedBy2),3)


        # check the geometry
        geometryFS = tGrid.extractCellsGeometry(fill=True, sort1Cells=True)
        geometryF  = tGrid.extractCellsGeometry(fill=True, sort1Cells=False)
        geometryS  = tGrid.extractCellsGeometry(fill=False, sort1Cells=True)
        geometry   = tGrid.extractCellsGeometry(fill=False, sort1Cells=False)
        geos = [geometryFS,geometryF,geometryS,geometry]
        for geo in geos:

            for c in [0,1,2]:
               g = geo[c]
               assert len(g) == numberOfCells[c]

    def test_corner_case_3x3_grid_b(self):

        assertEq = self.assertEqual


        seg = [
            [1,1,1],
            [1,2,1],
            [1,1,1]
        ]

        seg = numpy.array(seg,dtype='uint32')
        tGrid = ncgp.TopologicalGrid2D(seg)

        numberOfCells  = tGrid.numberOfCells
        assertEq(numberOfCells,[0,1,2])

        tShape = tGrid.topologicalGridShape
        assertEq(tShape, [5,5])

        shape = tGrid.shape
        assertEq(shape, [3,3])


        # check the bounds
        bounds = tGrid.extractCellsBounds()

        bounds0 = bounds[0]
        bounds1 = bounds[1]
        boundedBy1 = bounds0.reverseMapping()
        boundedBy2 = bounds1.reverseMapping()

        assertEq(len(bounds0),0)
        assertEq(len(bounds1),1)
        assertEq(len(boundedBy1),1)
        assertEq(len(boundedBy2),2)


        # check the geometry
        geometryFS = tGrid.extractCellsGeometry(fill=True, sort1Cells=True)
        geometryF  = tGrid.extractCellsGeometry(fill=True, sort1Cells=False)
        geometryS  = tGrid.extractCellsGeometry(fill=False, sort1Cells=True)
        geometry   = tGrid.extractCellsGeometry(fill=False, sort1Cells=False)


        geos = [geometryFS,geometryF,geometryS,geometry]
        for geo in geos:

            for c in [0,1,2]:
               g = geo[c]
               assert len(g) == numberOfCells[c]

    def test_corner_case_3x3_grid_c(self):

        assertEq = self.assertEqual


        seg = [
            [1,1,3],
            [1,2,3],
            [1,1,3]
        ]

        seg = numpy.array(seg,dtype='uint32')
        tGrid = ncgp.TopologicalGrid2D(seg)

        numberOfCells  = tGrid.numberOfCells
        assertEq(numberOfCells,[2,4,3])

        tShape = tGrid.topologicalGridShape
        assertEq(tShape, [5,5])

        shape = tGrid.shape
        assertEq(shape, [3,3])


        # check the bounds
        bounds = tGrid.extractCellsBounds()

        bounds0 = bounds[0]
        bounds1 = bounds[1]
        boundedBy1 = bounds0.reverseMapping()
        boundedBy2 = bounds1.reverseMapping()

        assertEq(len(bounds0),2)
        assertEq(len(bounds1),4)
        assertEq(len(boundedBy1),4)
        assertEq(len(boundedBy2),3)


        # check the geometry
        geometryFS = tGrid.extractCellsGeometry(fill=True, sort1Cells=True)
        geometryF  = tGrid.extractCellsGeometry(fill=True, sort1Cells=False)
        geometryS  = tGrid.extractCellsGeometry(fill=False, sort1Cells=True)
        geometry   = tGrid.extractCellsGeometry(fill=False, sort1Cells=False)


        geos = [geometryFS,geometryF,geometryS,geometry]
        for geo in geos:

            for c in [0,1,2]:
               g = geo[c]
               assert len(g) == numberOfCells[c]

    def test_corner_case_3x3_grid_d(self):

        assertEq = self.assertEqual


        seg = [
            [1,1,1],
            [1,2,1],
            [1,1,3]
        ]

        #     01234
        #   --------------------
        # 0  |1|1|1| 0
        # 1  |-*-*-| 1
        # 2  |1|2|1| 2
        # 3  |-*-*-| 3
        # 4  |1|1|3| 4
        # ----------------------
        #     01234

        seg = numpy.array(seg,dtype='uint32').T
        tGrid = ncgp.TopologicalGrid2D(seg)

        numberOfCells  = tGrid.numberOfCells
        assertEq(numberOfCells,[0,2,3])

        tShape = tGrid.topologicalGridShape
        assertEq(tShape, [5,5])

        shape = tGrid.shape
        assertEq(shape, [3,3])


        # check the bounds
        bounds = tGrid.extractCellsBounds()

        bounds0 = bounds[0]
        bounds1 = bounds[1]
        boundedBy1 = bounds0.reverseMapping()
        boundedBy2 = bounds1.reverseMapping()

        assertEq(len(bounds0),0)
        assertEq(len(bounds1),2)
        assertEq(len(boundedBy1),2)
        assertEq(len(boundedBy2),3)


        # check the geometry
        #print(seg)
        geometryFS = tGrid.extractCellsGeometry(fill=True, sort1Cells=True)



        geometryF  = tGrid.extractCellsGeometry(fill=True, sort1Cells=False)
        geometryS  = tGrid.extractCellsGeometry(fill=False, sort1Cells=True)
        geometry   = tGrid.extractCellsGeometry(fill=False, sort1Cells=False)
        geos = [geometryFS,geometryF,geometryS,geometry]
        for geo in geos:

            for c in [0,1,2]:
               g = geo[c]
               assert len(g) == numberOfCells[c]

    def test_randomized_big(self):

        for x in  range(100):
            assertEq = self.assertEqual
            shape = (10, 20)
            size = shape[0]*shape[1]
            labels = numpy.random.randint(0, 4,size=size).reshape(shape)

            gg = nifty.graph.undirectedGridGraph(shape)
            cc = nifty.graph.connectedComponentsFromNodeLabels(gg, labels.ravel())
            cc = cc.reshape(shape) + 1
            cc = numpy.require(cc, dtype='uint32')

            tGrid = ncgp.TopologicalGrid2D(cc)
            numberOfCells = tGrid.numberOfCells

            assertEq(numberOfCells[2], cc.max())

            # check the bounds
            bounds = tGrid.extractCellsBounds()
            boundedBy = {
                1:bounds[0].reverseMapping(),
                2:bounds[1].reverseMapping(),
            }
            try:
                # check the geometry
                geometryFS = tGrid.extractCellsGeometry(fill=True, sort1Cells=True)
                geometryF  = tGrid.extractCellsGeometry(fill=True, sort1Cells=False)
                geometryS  = tGrid.extractCellsGeometry(fill=False, sort1Cells=True)
                geometry   = tGrid.extractCellsGeometry(fill=False, sort1Cells=False)
                geos = [geometryFS,geometryF,geometryS,geometry]
            except:
                print(cc)
                import sys
                sys.exit()

    def test_randomized_medium(self):

        for x in  range(1000):
            assertEq = self.assertEqual
            shape = (7, 7)
            size = shape[0]*shape[1]
            labels = numpy.random.randint(0, 4,size=size).reshape(shape)

            #print(labels)

            gg = nifty.graph.undirectedGridGraph(shape)
            cc = nifty.graph.connectedComponentsFromNodeLabels(gg, labels.ravel())
            cc = cc.reshape(shape) + 1
            cc = numpy.require(cc, dtype='uint32')

            tGrid = ncgp.TopologicalGrid2D(cc)
            numberOfCells  = tGrid.numberOfCells

            assertEq(numberOfCells[2], cc.max())


            # check the bounds
            bounds = tGrid.extractCellsBounds()
            boundedBy = {
                1:bounds[0].reverseMapping(),
                2:bounds[1].reverseMapping(),
            }
            try:
                # check the geometry
                geometryFS = tGrid.extractCellsGeometry(fill=True, sort1Cells=True)
                geometryF  = tGrid.extractCellsGeometry(fill=True, sort1Cells=False)
                geometryS  = tGrid.extractCellsGeometry(fill=False, sort1Cells=True)
                geometry   = tGrid.extractCellsGeometry(fill=False, sort1Cells=False)
                geos = [geometryFS,geometryF,geometryS,geometry]
            except:
                print(cc)
                import sys
                sys.exit()

    def test_randomized_small(self):

        for x in  range(3000):
            assertEq = self.assertEqual
            shape = (4, 3)
            size = shape[0]*shape[1]
            labels = numpy.random.randint(1, 5,size=size).reshape(shape)

            #print(labels)

            gg = nifty.graph.undirectedGridGraph(shape)
            cc = nifty.graph.connectedComponentsFromNodeLabels(gg, labels.ravel())
            cc = cc.reshape(shape) + 1
            cc = numpy.require(cc, dtype='uint32')

            tGrid = ncgp.TopologicalGrid2D(cc)
            numberOfCells  = tGrid.numberOfCells

            assertEq(numberOfCells[2], cc.max())


            # check the bounds
            bounds = tGrid.extractCellsBounds()
            boundedBy = {
                1:bounds[0].reverseMapping(),
                2:bounds[1].reverseMapping(),
            }
            try:
                # check the geometry
                geometryFS = tGrid.extractCellsGeometry(fill=True, sort1Cells=True)
                geometryF  = tGrid.extractCellsGeometry(fill=True, sort1Cells=False)
                geometryS  = tGrid.extractCellsGeometry(fill=False, sort1Cells=True)
                geometry   = tGrid.extractCellsGeometry(fill=False, sort1Cells=False)
                geos = [geometryFS,geometryF,geometryS,geometry]
            except:
                print(cc)
                print("labels")
                print(labels)
                import sys
                sys.exit()


if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import
from . import _agglo as __agglo
from ._agglo import *

import numpy

__all__ = []
for key in __agglo.__dict__.keys():
    __all__.append(key)
    try:
        __agglo.__dict__[key].__module__='nifty.graph.agglo'
    except:
        pass

from ...tools import makeDense as __makeDense


def updateRule(name, **kwargs):
    if name in ['max', 'single_linkage']:
        return MaxSettings()
    elif name in ['mutex_watershed', 'abs_max']:
        return MutexWatershedSettings()
    elif name in ['min', 'complete_linkage']:
        return MinSettings()
    elif name == 'sum':
        return SumSettings()
    elif name in ['mean', 'average', 'avg']:
        return ArithmeticMeanSettings()
    elif name in ['gmean', 'generalized_mean']:
        p = kwargs.get('p',1.0)
        return GeneralizedMeanSettings(p=float(p))
    elif name in ['smax', 'smooth_max']:
        p = kwargs.get('p',0.0)
        return SmoothMaxSettings(p=float(p))
    elif name in ['rank','quantile', 'rank_order']:
        q = kwargs.get('q',0.5)
        numberOfBins = kwargs.get('numberOfBins',40)
        return RankOrderSettings(q=float(q), numberOfBins=int(numberOfBins))
    else:
        return NotImplementedError("not yet implemented")

def get_GASP_policy(graph,
                    signed_edge_weights,
                    linkage_criteria = 'mean',
                    linkage_criteria_kwargs = None,
                    add_cannot_link_constraints= False,
                    edge_sizes = None,
                    is_mergeable_edge = None,
                    node_sizes = None,
                    size_regularizer = 0.0,
                    number_of_nodes_to_stop = 1,
                    merge_constrained_edges_at_the_end=False,
                    collect_stats_for_exported_data=False
                    ):
    linkage_criteria_kwargs = {} if linkage_criteria_kwargs is None else linkage_criteria_kwargs
    parsed_rule = updateRule(linkage_criteria, **linkage_criteria_kwargs)

    edge_sizes = numpy.ones_like(signed_edge_weights) if edge_sizes is None else edge_sizes
    is_mergeable_edge = numpy.ones_like(signed_edge_weights) if is_mergeable_edge is None else is_mergeable_edge
    node_sizes = numpy.ones(graph.numberOfNodes ,dtype='float32') if node_sizes is None else node_sizes

    return gaspClusterPolicy(graph=graph,
                             signedWeights=signed_edge_weights,
                             isMergeEdge=is_mergeable_edge,
                             edgeSizes=edge_sizes,
                             nodeSizes=node_sizes,
                             updateRule0=parsed_rule,
                             numberOfNodesStop=number_of_nodes_to_stop,
                             sizeRegularizer=size_regularizer,
                             addNonLinkConstraints=add_cannot_link_constraints,
                             mergeConstrainedEdgesAtTheEnd=merge_constrained_edges_at_the_end,
                             collectStats=collect_stats_for_exported_data)


get_GASP_policy.__doc__ = """
GASP: generalized agglomeration for signed graph partition

Accepted update rules:
 - 'mean'
 - 'max' (single linkage)
 - 'min' (complete linkage)
 - 'MutexWatershed' (abs-max)
 - 'sum'
 - {name: 'rank', q=0.5, numberOfBins=40}
 - {name: 'generalized_mean', p=2.0}   # 1.0 is mean
 - {name: 'smooth_max', p=2.0}   # 0.0 is mean
 """





def sizeLimitClustering(graph, nodeSizes, minimumNodeSize,
                        edgeIndicators=None,edgeSizes=None,
                        sizeRegularizer=0.001, gamma=0.999,
                        makeDenseLabels=False):

    s = graph.edgeIdUpperBound + 1

    def rq(data):
        return numpy.require(data, 'float32')

    nodeSizes  = rq(nodeSizes)

    if edgeIndicators is None:
        edgeIndicators = numpy.ones(s,dtype='float32')
    else:
        edgeIndicators = rq(edgeIndicators)

    if edgeSizes is None:
        edgeSizes = numpy.ones(s,dtype='float32')
    else:
        edgeSizes = rq(edgeSizes)



    cp =  minimumNodeSizeClusterPolicy(graph, edgeIndicators=edgeIndicators,
                                              edgeSizes=edgeSizes,
                                              nodeSizes=nodeSizes,
                                              minimumNodeSize=float(minimumNodeSize),
                                              sizeRegularizer=float(sizeRegularizer),
                                              gamma=float(gamma))

    agglo = agglomerativeClustering(cp)

    agglo.run()
    labels = agglo.result()

    if makeDenseLabels:
        labels = __makeDense(labels)

    return labels;




def ucmFeatures(graph, edgeIndicators, edgeSizes, nodeSizes,
                sizeRegularizers = numpy.arange(0.1,1,0.1) ):

    def rq(data):
        return numpy.require(data, 'float32')

    edgeIndicators = rq(edgeIndicators)

    if edgeSizes is None:
        edgeSizes = numpy.ones(graph.numberOfEdges, dtype='float32')
    else:
        edgeSizes = rq(edgeSizes)


    if nodeSizes is None:
        nodeSizes = numpy.ones(graph.numberOfNodes, dtype='float32')
    else:
        nodeSizes = rq(nodeSizes)

    fOut = []
    # policy
    for sr in sizeRegularizers:

        sr = float(sr)
        cp = edgeWeightedClusterPolicyWithUcm(graph=graph, edgeIndicators=edgeIndicators,
                edgeSizes=edgeSizes, nodeSizes=nodeSizes, sizeRegularizer=sr)


        agglo = agglomerativeClustering(cp)



        hA = agglo.runAndGetDendrogramHeight()[:,None]
        hB = agglo.ucmTransform(cp.edgeIndicators)[:,None]

        fOut.extend([hA,hB])

    return numpy.concatenate(fOut, axis=1)



import unittest

from nifty.graph import run_label_propagation, UndirectedGraph
import numpy as np


class TestLabelPropagation(unittest.TestCase):

    def test_label_prop_one_iter(self):
        # Test on a simple star-shaped graph:
        graph = UndirectedGraph(5)
        uv_ids = np.array([
            [0, 1],
            [0, 2],
            [0, 3],
            [0, 4],
        ])

        # Use some signed edge weights:
        edge_weights = np.array([
            1.,
            1.,
            -1.,
            -1.,
        ])
        graph.insertEdges(uv_ids)

        node_labels = run_label_propagation(graph, edge_values=edge_weights, nb_iter=1, local_edges=None,
                              size_constr=-1,
                              nb_threads=1)

        self.assertIn(node_labels[0], [node_labels[1], node_labels[2]])

    def test_label_prop_connected_components(self):
        # Test on a simple two-components graph:
        graph = UndirectedGraph(5)
        uv_ids = np.array([
            [0, 1],
            [0, 2],
            [1, 2],
            [3, 4],
        ])
        graph.insertEdges(uv_ids)

        node_labels = run_label_propagation(graph, nb_iter=30)

        self.assertNotEqual(node_labels[0],node_labels[3])
        self.assertEqual(node_labels[0],node_labels[1])
        self.assertEqual(node_labels[0],node_labels[2])
        self.assertEqual(node_labels[3],node_labels[4])


if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import
from __future__ import division

import numpy
import types
from functools import partial
import numpy
import time
import sys
import warnings


try:
    _hasVolumina=True
    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        import volumina
        from volumina.api import Viewer
except:
    _hasVolumina=False



try:
    _hasVigra=True
    import vigra
except:
    _hasVigra=False


try:
    _hasQt4=True
    from PyQt4.QtGui import QApplication
except:
    _hasQt4=False





def view3D(img, show=True, cmap='Greys',title=None, singlePlain=True):

    import pylab
    import matplotlib.cm as cm

    shape = img.shape

    f = pylab.figure()

    if singlePlain:
        pylab.imshow(img[shape[0]//2,:,:],cmap=cmap)
    else:
        f.add_subplot(2, 2, 1)
        pylab.imshow(img[shape[0]//2,:,:],cmap=cmap)

        f.add_subplot(2, 2, 2)
        pylab.imshow(img[:,shape[1]//2,:],cmap=cmap)

        f.add_subplot(2, 2, 3)
        pylab.imshow(img[:,:,shape[2]//2],cmap=cmap)

    if title is not None:
        pylab.title(title)

    if show:
        pylab.show()



class Singleton:
    """
    A non-thread-safe helper class to ease implementing singletons.
    This should be used as a decorator -- not a metaclass -- to the
    class that should be a singleton.
    The decorated class can define one `__init__` function that
    takes only the `self` argument. Other than that, there are
    no restrictions that apply to the decorated class.
    To get the singleton instance, use the `Instance` method. Trying
    to use `__call__` will result in a `TypeError` being raised.
    Limitations: The decorated class cannot be inherited from.
    """

    def __init__(self, decorated):
        self._decorated = decorated

    def Instance(self):
        """
        Returns the singleton instance. Upon its first call, it creates a
        new instance of the decorated class and calls its `__init__` method.
        On all subsequent calls, the already created instance is returned.
        """
        try:
            return self._instance
        except AttributeError:
            self._instance = self._decorated()
            return self._instance

    def __call__(self):
        raise TypeError('Singletons must be accessed through `Instance()`.')

    def __instancecheck__(self, inst):
        return isinstance(inst, self._decorated)


if _hasQt4:
    @Singleton
    class QApp:
        def __init__(self):
            self.app = QApplication(sys.argv)










if _hasQt4 and _hasVolumina:
    def addHocViewer(grayData=None, segData=None, title="viewer",visu=True):
        if visu :
            app = QApp.Instance().app

            v = Viewer()

            if grayData is not None:
                for name in grayData.keys():
                    data = grayData[name]
                    if _hasVigra:
                        if isinstance(data, vigra.arraytypes.VigraArray):
                            v.addGrayscaleLayer(data.view(numpy.ndarray), name=name)
                        else:
                            v.addGrayscaleLayer(data, name=name)
                    else:
                        v.addGrayscaleLayer(data, name=name)

            if segData is not None:
                for name in segData.keys():
                    data = segData[name]
                    if _hasVigra:
                        if isinstance(data, vigra.arraytypes.VigraArray):
                            v.addColorTableLayer(data.view(numpy.ndarray), name=name)
                        else:
                            v.addColorTableLayer(data, name=name)
                    else:
                        v.addGrayscaleLayer(data, name=name)

            v.setWindowTitle(title)
            v.showMaximized()
            app.exec_()



from __future__ import absolute_import,print_function
from . import _ground_truth as __ground_truth
from ._ground_truth import *

import numpy
from scipy import ndimage as ndi

__all__ = []
for key in __ground_truth.__dict__.keys():
    __all__.append(key)

    try:
        __ground_truth.__dict__[key].__module__='nifty.ground_truth'
    except:
        pass




def thinSegFilter(seg, sigma, radius=None):
    edges = segToEdges2D(seg)
    dt = ndi.distance_transform_edt(1 - edges)
    if radius is None:
        radius = int(3.5*float(sigma) + 0.5)
        radius += 4


    p_seg = numpy.pad(seg,radius,mode='reflect').astype('uint32')
    p_dt  = numpy.pad(dt,radius ,mode='reflect').astype('uint32')

    print("p_seg",p_seg.shape,p_seg.dtype)
    print("p_dt",p_dt.shape,p_dt.dtype)
    print("sigma",sigma)
    print("radius",radius)

    out = _ground_truth._thinSegFilter(p_seg, p_dt, sigma=float(sigma), radius=radius)
    return out[radius:radius+seg.shape[0],radius:radius+seg.shape[1]],dt




def overlap(segmentation, groundTruth):
    """factory function for :class:`nifty.ground_truth.Overlap`

    create an instance of :class:`nifty.ground_truth.Overlap`
    which can be used to project ground truth
    to some segmentation / over-segmentation

    Args:
        segmentation (numpy.ndarray): The segmentation / over-segmentation
        groundTruth (numpy.ndarray): The ground truth as node labeling.


    Returns:
        [description]
        [type]
    """
    a = numpy.require(segmentation, dtype='uint64')
    b = numpy.require(groundTruth, dtype='uint64')
    return Overlap(a.max(), a, b)



from __future__ import absolute_import
from .import _lifted_multicut as __lifted_multicut
from ._lifted_multicut import *
from functools import partial
from ..multicut import ilpSettings
from .. import Configuration
import numpy

__all__ = []
for key in __lifted_multicut.__dict__.keys():
    __all__.append(key)

    try:
        __lifted_multicut.__dict__[key].__module__='nifty.graph.opt.lifted_multicut'
    except:
        pass


class PixelWiseLmcObjective(object):
    def __init__(self, weights, offsets):

        self.weights = weights
        self.offsets = offsets

        if(self.offsets.shape[1] == 2):
            assert weights.shape[2] == offsets.shape[0]
            self.shape = weights.shape[0:2]
            self.n_variables = self.shape[0]*self.shape[1]
            self.ndim = 2
            self._obj = PixelWiseLmcObjective2D(self.weights, self.offsets)
        elif (self.offsets.shape[1] == 3):
            self.shape = weights.shape[0:3]
            self.n_variables = self.shape[0]*self.shape[1]*self.shape[2]
            self.ndim = 3
            assert weights.shape[3] == offsets.shape[0]
            self._obj = PixelWiseLmcObjective3D(self.weights, self.offsets)
        else:
            raise NotImplementedError("PixelWiseLmcObjective is only implemented for 2D and 3D images")

    def optimize(self,factory, labels=None):
        if labels is None:
            labels = numpy.arange(self.n_variables).reshape(self.shape)
        return self._obj.optimize(factory, labels)





    def evaluate(self, labels):
        return self._obj.evaluate(labels)

    def cpp_obj(self):
        return self._obj


def pixelWiseLmcObjective(weights, offsets):
    return PixelWiseLmcObjective(weights, offsets)








def __extendLiftedMulticutObj(objectiveCls, objectiveName):

    def insertLiftedEdgesBfs(self, maxDistance, returnDistance = False):
        if returnDistance :
            return self._insertLiftedEdgesBfsReturnDist(maxDistance)
        else:
            self._insertLiftedEdgesBfs(maxDistance)

    objectiveCls.insertLiftedEdgesBfs = insertLiftedEdgesBfs







    def getCls(prefix, postfix):
        return _lifted_multicut.__dict__[prefix+postfix]

    def getSettingsCls(baseName):
        S =  getCls(baseName + "SettingsType" ,objectiveName)
        return S
    def getLmcCls(baseName):
        S =  getCls(baseName,objectiveName)
        return S
    def getSettings(baseName):
        S =  getSettingsCls(baseName)
        return S()
    def getSettingsAndFactoryCls(baseName):
        s =  getSettings(baseName)
        F =  getCls(baseName + "Factory" ,objectiveName)
        return s,F

    def factoryClsName(baseName):
        return baseName + "Factory" + objectiveName


    O = objectiveCls

    def verboseVisitor(visitNth=1,
                       timeLimitSolver=float('inf'),
                       timeLimitTotal=float('inf')):
        V = getLmcCls("LiftedMulticutVerboseVisitor")
        return V(visitNth, float(timeLimitSolver), float(timeLimitTotal))
    O.verboseVisitor = staticmethod(verboseVisitor)



    def chainedSolversFactory(factories):
        s,F = getSettingsAndFactoryCls("ChainedSolvers")
        s.factories = factories
        return F(s)
    O.chainedSolversFactory = staticmethod(chainedSolversFactory)

    O.chainedSolversFactory.__doc__ = """ create an instance of :class:`%s`

        Chain multiple solvers
        such that each successor is warm-started with
        its predecessor solver.

    Warning:
        The solvers should be able to be warm started.

    Args:
        weightStopCond (float): stop clustering when the highest
            weight in cluster-graph is lower as this value (default: {0.0})
        nodeNumStopCond: stop clustering when a cluster-graph
            reached a certain number of nodes.
            Numbers smaller 1 are interpreted as fraction
            of the graphs number of nodes.
            If nodeNumStopCond is smaller 0 this
            stopping condition is ignored  (default: {-1})
    Returns:
        %s : multicut factory
    """%(factoryClsName("ChainedSolvers"),factoryClsName("ChainedSolvers"))


    def watershedProposalGenerator(sigma=1.0, numberOfSeeds=0.1,
                                   seedingStrategy='SEED_FROM_LIFTED'):
        """factory function for a watershed based proposal generator for fusion move based
        lifted multicuts.

        Args:
            sigma (float, optional): The weights are perturbed by a additive
                Gaussian noise n(0,sigma) (default  0.0)

            numberOfSeeds (float, optional): Number of seed to generate.
                A number smaller as one will be interpreted as a fraction of
                the number of nodes (default 0.1)
            seedingStrategy (str, optional): Can be:
                - 'SEED_FROM_LIFTED' : All negative weighted lifted edges
                    can be used to generate seeds.
                - 'SEED_FROM_LOCAL' : All negative weighted local edges
                    can be used to generate seeds.
                - 'SEED_FROM_BOTH' : Both, lifted and local edges
                can be used for seeding

        Returns:
            TYPE: parameter object used construct a WatershedProposalGenerator

        """
        pGenCls = getLmcCls("WatershedProposalGeneratorFactory")
        pGenSettings = getSettings("WatershedProposalGenerator")

        # map string to enum
        stringToEnum = {
            'SEED_FROM_LIFTED' : pGenSettings.SeedingStrategy.SEED_FROM_LIFTED,
            'SEED_FROM_LOCAL' : pGenSettings.SeedingStrategy.SEED_FROM_LOCAL,
            'SEED_FROM_BOTH' : pGenSettings.SeedingStrategy.SEED_FROM_BOTH,
        }
        try:
            enumVal = stringToEnum[seedingStrategy]
        except:
            raise RuntimeError("unkown seedingStrategy '%s': must be either"\
                               "'SEED_FROM_LIFTED','SEED_FROM_LOCAL' or "\
                               " 'SEED_FROM_BOTH' "%str(seedingStrategy))

        pGenSettings.sigma = float(sigma)
        pGenSettings.numberOfSeeds = float(numberOfSeeds)
        pGenSettings.seedingStrategy = enumVal

        return pGenCls(pGenSettings)

    O.watershedProposalGenerator = staticmethod(watershedProposalGenerator)


    def fusionMoveBasedFactory(proposalGenerator=None, numberOfThreads=1,
        numberOfIterations=1000, stopIfNoImprovement=100):
        """factory function for a fusion move based lifted
            multicut solver

        Args:
            proposalGenerator (None, optional): Proposal generator (default watershedProposalGenerator)
            numberOfThreads (int, optional):                (default 1)
            numberOfIterations (int, optional): Maximum number of iterations(default 1000)
            stopIfNoImprovement (int, optional): Stop after n iterations without improvement (default 100)

        Returns:
            TYPE: Description
        """
        if proposalGenerator is None:
            proposalGenerator = watershedProposalGenerator()
        s,F = getSettingsAndFactoryCls("FusionMoveBased")
        s.proposalGenerator = proposalGenerator
        s.numberOfThreads = int(numberOfThreads)
        s.numberOfIterations = int(numberOfIterations)
        s.stopIfNoImprovement = int(stopIfNoImprovement)
        return F(s)

    O.fusionMoveBasedFactory = staticmethod(fusionMoveBasedFactory)


    def liftedMulticutGreedyAdditiveFactory(weightStopCond=0.0, nodeNumStopCond=-1.0):
        s,F = getSettingsAndFactoryCls("LiftedMulticutGreedyAdditive")
        s.weightStopCond = float(weightStopCond)
        s.nodeNumStopCond = float(nodeNumStopCond)
        return F(s)
    O.liftedMulticutGreedyAdditiveFactory = staticmethod(liftedMulticutGreedyAdditiveFactory)


    def liftedMulticutKernighanLinFactory(numberOfOuterIterations=1000000,
                                          numberOfInnerIterations=100,
                                          epsilon=1e-7):
        s,F = getSettingsAndFactoryCls("LiftedMulticutKernighanLin")
        s.numberOfOuterIterations = int(numberOfOuterIterations)
        s.numberOfInnerIterations = int(numberOfInnerIterations)
        s.epsilon = float(epsilon)
        return F(s)
    O.liftedMulticutKernighanLinFactory = staticmethod(liftedMulticutKernighanLinFactory)


    if Configuration.WITH_LP_MP:
        def liftedMulticutMpFactory(
            lmcFactory = None,
            greedyWarmstart = False,
            tightenSlope = 0.05,
            tightenMinDualImprovementInterval = 0,
            tightenMinDualImprovement = 0.,
            tightenConstraintsPercentage = 0.1,
            tightenConstraintsMax = 0,
            tightenInterval = 10,
            tightenIteration = 100,
            tightenReparametrization = "anisotropic",
            roundingReparametrization = "anisotropic",
            standardReparametrization = "anisotropic",
            tighten = True,
            minDualImprovementInterval = 0,
            minDualImprovement = 0.,
            lowerBoundComputationInterval = 1,
            primalComputationInterval = 5,
            timeout = 0,
            maxIter = 1000,
            numThreads = 1
            ):

            s, F = getSettingsAndFactoryCls("LiftedMulticutMp")
            if lmcFactory is None:
                lmcFactory = LiftedMulticutObjectiveUndirectedGraph.liftedMulticutKernighanLinFactory()

            s.lmcFactory      = lmcFactory
            s.greedyWarmstart = greedyWarmstart
            return F(s)
        O.liftedMulticutMpFactory = staticmethod(liftedMulticutMpFactory)


    def liftedMulticutIlpFactory(verbose=0, addThreeCyclesConstraints=True,
                                addOnlyViolatedThreeCyclesConstraints=True,
                                relativeGap=0.0, absoluteGap=0.0, memLimit=-1.0,
                                ilpSolver = 'cplex'):

        if ilpSolver == 'cplex':
            s,F = getSettingsAndFactoryCls("LiftedMulticutIlpCplex")
        elif ilpSolver == 'gurobi':
            s,F = getSettingsAndFactoryCls("LiftedMulticutIlpGurobi")
        elif ilpSolver == 'glpk':
            s,F = getSettingsAndFactoryCls("LiftedMulticutIlpGlpk")
        else:
            raise RuntimeError("%s is an unknown ilp solver"%str(ilpSolver))
        s.verbose = int(verbose)
        s.addThreeCyclesConstraints = bool(addThreeCyclesConstraints)
        s.addOnlyViolatedThreeCyclesConstraints = bool(addOnlyViolatedThreeCyclesConstraints)
        s.ilpSettings = ilpSettings(relativeGap=relativeGap, absoluteGap=absoluteGap, memLimit=memLimit)
        return F(s)

    O.liftedMulticutIlpFactory = staticmethod(liftedMulticutIlpFactory)
    O.liftedMulticutIlpCplexFactory = staticmethod(partial(liftedMulticutIlpFactory,ilpSolver='cplex'))
    O.liftedMulticutIlpGurobiFactory = staticmethod(partial(liftedMulticutIlpFactory,ilpSolver='gurobi'))
    O.liftedMulticutIlpGlpkFactory = staticmethod(partial(liftedMulticutIlpFactory,ilpSolver='glpk'))


__extendLiftedMulticutObj(LiftedMulticutObjectiveUndirectedGraph,
    "LiftedMulticutObjectiveUndirectedGraph")

__extendLiftedMulticutObj(LiftedMulticutObjectiveUndirectedGridGraph2DSimpleNh,
    "LiftedMulticutObjectiveUndirectedGridGraph2DSimpleNh")

__extendLiftedMulticutObj(LiftedMulticutObjectiveUndirectedGridGraph3DSimpleNh,
    "LiftedMulticutObjectiveUndirectedGridGraph2DSimpleNh")

del __extendLiftedMulticutObj



from __future__ import print_function
import nifty
import numpy
import nifty
import tempfile
import shutil
import os 

from nose.tools import assert_almost_equals

nrag = nifty.graph.rag


def ensureDir(f):
    d = os.path.dirname(f)
    if not os.path.exists(d):
        os.makedirs(d)


def testAccumulateEdgeMeanAndLengthRag2d():

    labels = numpy.array( [
        [0,1,2],
        [0,0,2],
        [3,3,2],
        [4,4,4]
    ]).T

    data = numpy.array([
        [1,1,1],
        [1,1,1],
        [1,1,1],
        [1,1,1]
    ], dtype='float32').T

    rag = nrag.gridRag(labels)

    features = nrag.accumulateEdgeMeanAndLength(rag, data, [2,2], 1)


    edgesAndCounts = [
        ((0,1), 2*2 ),
        ((0,2), 1*2 ),
        ((0,3), 2*2 ),
        ((1,2), 1*2 ),
        ((2,3), 1*2 ),
        ((2,4), 1*2 ),
        ((3,4), 2*2 )
    ]

    mean = features[:,0]
    count = features[:,1]

    for edge,c in edgesAndCounts:

        edgeId = rag.findEdge(edge[0], edge[1])

        #print (edge, "myc",count[edgeId],"shouldc",c)
        assert count[edgeId] == c



def testAccumulateMeanAndLengthRag2d():

    labels = numpy.array( [
        [0,  1, 2],
        [0,  0, 2],
        [3,  3, 2],
        [4,  4, 4]
    ]).T

    data = numpy.array([
        [1,  2, 3],
        [4,  5, 6],
        [7,  8, 9],
        [10,11,12]
    ], dtype='float32').T

    rag = nrag.gridRag(labels)

    eFeatures, nFeatures = nrag.accumulateMeanAndLength(rag, data, [2,2], -1)


    edgesAndCountsAndMeans = [
        ((0,1), 2*2, float(1+2+2+5)/4 ),
        ((0,2), 1*2, float(5+6)/2 ),
        ((0,3), 2*2, float(4+5+7+8)/4 ),
        ((1,2), 1*2, float(2+3)/2 ),
        ((2,3), 1*2, float(8+9)/2 ),
        ((2,4), 1*2, float(9+12)/2 ),
        ((3,4), 2*2, float(7+8+10+11)/4 )
    ]

    nodeAndCountsAndMeans = [
        (0,3, (1.0+4.0+5.0)/3.0    ),
        (1,1, (2.0)/1.0            ),
        (2,3, (3.0+6.0+9.0)/3.0    ),
        (3,2, (7.0+8.0)/2.0        ),
        (4,3, (10.0+11.0+12.0)/3.0 )
    ]

    eMean = eFeatures[:,0]
    eCount = eFeatures[:,1]

    nMean = nFeatures[:,0]
    nCount = nFeatures[:,1]

    for edge,c,m in edgesAndCountsAndMeans:
        edgeId = rag.findEdge(edge[0], edge[1])
        assert eCount[edgeId] == c
        assert_almost_equals(m, eMean[edgeId],5)



    for node,c,m in nodeAndCountsAndMeans:
        assert nCount[node] == c
    for node,c,m in nodeAndCountsAndMeans:
        assert_almost_equals(m, nMean[node],5)


from ._carving import *


def carvingSegmenter(rag, edgeWeights, sortEdges=True):
    ndim = len(rag.shape)
    if ndim == 2:
        return CarvingSegmenterRag2D(rag, edgeWeights, sortEdges)
    else:
        return CarvingSegmenterRag3D(rag, edgeWeights, sortEdges)



from __future__ import absolute_import

from . import _mincut as __mincut
from ._mincut import *
from .... import Configuration

from functools import partial

__all__ = []
for key in __mincut.__dict__.keys():
    __all__.append(key)
    try:
        __mincut.__dict__[key].__module__='nifty.graph.opt.mincut'
    except:
        pass


def __extendMincutObj(objectiveCls, objectiveName):

    def getCls(prefix, postfix):
        return _mincut.__dict__[prefix+postfix]

    def getSettingsCls(baseName):
        S =  getCls(baseName + "SettingsType" ,objectiveName)
        return S
    def getSolverCls(baseName):
        S =  getCls(baseName,objectiveName)
        return S
    def getSettings(baseName):
        S =  getSettingsCls(baseName)
        return S()
    def getSettingsAndFactoryCls(baseName):
        s =  getSettings(baseName)
        F =  getCls(baseName + "Factory" ,objectiveName)
        return s,F

    O = objectiveCls

    def mincutVerboseVisitor(visitNth=1,timeLimit=0):
        V = getSolverCls("VerboseVisitor")
        return V(visitNth,timeLimit)
    #O.mincutVerboseVisitor = staticmethod(mincutVerboseVisitor)
    O.verboseVisitor = staticmethod(mincutVerboseVisitor)


    def watershedProposalGenerator(sigma=1.0, numberOfSeeds=0.1,seedingStrategie='SEED_FROM_NEGATIVE'):
        """factory function for a watershed based proposal generator for mincutCcFusionMoveBased

        Args:
            sigma (float, optional): The weights are perturbed by a additive
                Gaussian noise n(0,sigma) (default  0.0)

            numberOfSeeds (float, optional): Number of seed to generate.
                A number smaller as one will be interpreted as a fraction of
                the number of nodes (default 0.1)
            seedingStrategie (str, optional): Can be:
                - 'SEED_FROM_NEGATIVE' : All negative weighted  edges
                    can be used to generate seeds.
                - 'SEED_FROM_ALL' : All edges
                    can be used to generate seeds.
        Returns:
            TYPE: parameter object used construct a WatershedProposalGenerator
        """
        pGenCls = getSolverCls("WatershedProposalGeneratorFactory")
        pGenSettings = getSettings("WatershedProposalGenerator")

        # map string to enum
        stringToEnum = {
            'SEED_FROM_NEGATIVE' : pGenSettings.SeedingStrategie.SEED_FROM_NEGATIVE,
            'SEED_FROM_ALL' :      pGenSettings.SeedingStrategie.SEED_FROM_ALL,
        }
        try:
            enumVal = stringToEnum[seedingStrategie]
        except:
            raise RuntimeError("unkown seedingStrategie '%s': must be either"\
                               "'SEED_FROM_NEGATIVE' or 'SEED_FROM_ALL'"%str(seedingStrategie))

        pGenSettings.sigma = float(sigma)
        pGenSettings.numberOfSeeds = float(numberOfSeeds)
        pGenSettings.seedingStrategie = enumVal

        return pGenCls(pGenSettings)

    O.watershedProposalGenerator = staticmethod(watershedProposalGenerator)


    def mincutQpboFactory(improve=True):
        if Configuration.WITH_QPBO:
            s,F = getSettingsAndFactoryCls("MincutQpbo")
            s.improve = bool(improve)
            return F(s)
        else:
            raise RuntimeError("mincutQpbo need nifty to be compiled WITH_QPBO")
    O.mincutQpboFactory = staticmethod(mincutQpboFactory)


    def greedyAdditiveFactory( weightStopCond=0.0, nodeNumStopCond=-1.0, improve=True):
        if Configuration.WITH_QPBO:
            s,F = getSettingsAndFactoryCls("MincutGreedyAdditive")
            s.weightStopCond = float(weightStopCond)
            s.nodeNumStopCond = float(nodeNumStopCond)
            s.improve = bool(improve)
            return F(s)
        else:
            raise RuntimeError("greedyAdditiveFactory need nifty to be compiled WITH_QPBO")
    O.greedyAdditiveFactory = staticmethod(greedyAdditiveFactory)


    def mincutCcFusionMoveSettings(mincutFactory=None):
        if mincutFactory is None:
            if Configuration.WITH_QPBO:
                mincutFactory = MincutObjectiveUndirectedGraph.mincutQpboFactory()
            else:
                raise RuntimeError("default mincutFactory needs mincutQpbo, which need nifty to be compiled WITH_QPBO")

        s = getSettings("MincutCcFusionMove")
        s.mincutFactory = mincutFactory
        return s


    def mincutCcFusionMoveBasedFactory(proposalGenerator=None, numberOfThreads=1,
        numberOfIterations=1000, stopIfNoImprovement=100,
        fusionMoveSettings=None):
        """factory function for a  cc-fusion move based mincut solver

        Args:
            proposalGenerator (None, optional): Proposal generator (default watershedProposalGenerator)
            numberOfThreads (int, optional):                (default 1)
            numberOfIterations (int, optional): Maximum number of iterations(default 1000)
            stopIfNoImprovement (int, optional): Stop after n iterations without improvement (default 100)
            fusionMoveSettings (FusionMoveSettings, optional) : The settings of the underlaying mincutCcFusionMove
        Returns:
            TYPE: MincutCcFusionMoveBasedFactory
        """
        if proposalGenerator is None:
            proposalGenerator = watershedProposalGenerator()

        if fusionMoveSettings is None:
            fusionMoveSettings = mincutCcFusionMoveSettings()

        s,F = getSettingsAndFactoryCls("MincutCcFusionMoveBased")
        s.proposalGenerator = proposalGenerator
        s.numberOfThreads = int(numberOfThreads)
        s.numberOfIterations = int(numberOfIterations)
        s.stopIfNoImprovement = int(stopIfNoImprovement)
        s.fusionMoveSettings = fusionMoveSettings
        return F(s)

    O.mincutCcFusionMoveBasedFactory = staticmethod(mincutCcFusionMoveBasedFactory)


__extendMincutObj(MincutObjectiveUndirectedGraph,
    "MincutObjectiveUndirectedGraph")
__extendMincutObj(MincutObjectiveEdgeContractionGraphUndirectedGraph,
    "MincutObjectiveEdgeContractionGraphUndirectedGraph")
del __extendMincutObj



import unittest


import numpy as np
import nifty.graph.rag as nrag
import nifty.graph as ngraph


# Just a quick and dirty test to check that the functions can be called
class TestAccumulate(unittest.TestCase):
    shape_2d = 2 * (128,)
    shape_3d = 3 * (64,)

    def test_lifted_edges_2d(self):
        offsets = [
            [1, 2], # One down, two right
            [2, 0]  # Two down
        ]
        labels = np.array([
            [0, 1, 2],
            [0, 1, 1],
            [0, 1, 3]
        ], dtype='uint32')
        rag = nrag.gridRag(labels, numberOfLabels=4)
        lifted_edges = nrag.compute_lifted_edges_from_rag_and_offsets(rag, offsets)
        self.assertTrue(len(lifted_edges) == 2)
        for lifted_e in lifted_edges:
            self.assertTrue(3 in lifted_e)
            self.assertTrue(2 in lifted_e or 0 in lifted_e)



    def test_accumulate_mean_and_length_3d(self):
        labels = np.random.randint(0, 100, size=self.shape_3d, dtype='uint32')
        # rag = nrag.gridRag(labels, numberOfLabels=100)
        offsets = [
            [2, 20, -7],
            [0, 0,  -5]
        ]

        affinities = np.random.random(size=(2,) + self.shape_3d)

        edge_indicators_mean, edge_sizes = ngraph.accumulate_affinities_mean_and_length(affinities, offsets, labels)



if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
import shutil
import os
import unittest
import numpy
import nifty
import nifty.graph.rag as nrag


class TestRagBase(unittest.TestCase):

    def setUp(self):
        self.tmp = './tmp'
        if not os.path.exists(self.tmp):
            os.mkdir(self.tmp)
        self.path = os.path.join(self.tmp, 'rag')
        self.path2 = os.path.join(self.tmp, 'rag2')

    def tearDown(self):
        if os.path.exists(self.tmp):
            shutil.rmtree(self.tmp)

    def generic_rag_test(self, rag, numberOfNodes, shouldEdges, shouldNotEdges, shape=None):
        self.assertEqual(rag.numberOfNodes, numberOfNodes)
        self.assertEqual(rag.numberOfEdges, len(shouldEdges))
        if shape is not None:
            self.assertEqual(rag.shape, shape)

        edgeList = []
        for edge in rag.edges():
            edgeList.append(edge)

        assert len(edgeList) == len(shouldEdges)

        for shouldEdge in shouldEdges:

            fRes = rag.findEdge(shouldEdge)
            self.assertGreaterEqual(fRes, 0)
            uv = rag.uv(fRes)
            uv = sorted(uv)
            self.assertEqual(uv[0], shouldEdge[0])
            self.assertEqual(uv[1], shouldEdge[1])

        for shouldNotEdge in shouldNotEdges:
            fRes = rag.findEdge(shouldNotEdge)
            self.assertEqual(fRes, -1)


class TestRag(TestRagBase):
    # This will fail because the expliecit labels python bindings are broken
    def test_insert(self):
        labels = numpy.zeros(shape=(2, 2), dtype='uint32')

        labels[0, 0] = 0
        labels[1, 0] = 1
        labels[0, 1] = 0
        labels[1, 1] = 2

        g = nrag.gridRag(labels, int(labels.max() + 1))
        self.assertEqual(g.numberOfNodes, 3)
        self.assertEqual(g.numberOfEdges, 3)
        with self.assertRaises(RuntimeError):
            g.insertEdge(0, 1)

    # This will fail because the expliecit labels python bindings are broken
    def test_explicit_labels_rag2d(self):
        labels = numpy.array([[0, 1, 2],
                              [0, 0, 2],
                              [3, 3, 2],
                              [4, 4, 4]], dtype='uint32')

        n_labels = labels.max() + 1
        ragA = nifty.graph.rag.gridRag(labels, n_labels, numberOfThreads=1)
        ragB = nifty.graph.rag.gridRag(labels, n_labels)

        self.assertTrue(isinstance(ragA, nifty.graph.rag.ExplicitLabelsGridRag2D))
        self.assertTrue(isinstance(ragB, nifty.graph.rag.ExplicitLabelsGridRag2D))

        shouldEdges = [(0, 1),
                       (0, 2),
                       (0, 3),
                       (1, 2),
                       (2, 3),
                       (2, 4),
                       (3, 4)]

        shouldNotEdges = [(0, 4),
                          (1, 3),
                          (1, 4)]

        self.generic_rag_test(rag=ragA,
                              numberOfNodes=5,
                              shouldEdges=shouldEdges,
                              shouldNotEdges=shouldNotEdges)

        self.generic_rag_test(rag=ragB,
                              numberOfNodes=5,
                              shouldEdges=shouldEdges,
                              shouldNotEdges=shouldNotEdges)

    # This will fail because the expliecit labels python bindings are broken
    def test_explicit_labels_rag3d(self):
        labels = [[[0, 1],
                   [0, 0]],
                  [[1, 1],
                   [2, 2]],
                  [[3, 3],
                   [3, 3]]]
        labels = numpy.array(labels, dtype='uint32')

        n_labels = labels.max() + 1
        ragA = nifty.graph.rag.gridRag(labels, n_labels, numberOfThreads=1)
        ragB = nifty.graph.rag.gridRag(labels, n_labels)

        self.assertTrue(isinstance(ragA, nifty.graph.rag.ExplicitLabelsGridRag3D32))
        self.assertTrue(isinstance(ragB, nifty.graph.rag.ExplicitLabelsGridRag3D32))

        shouldEdges = [(0, 1),
                       (0, 2),
                       (1, 2),
                       (1, 3),
                       (2, 3)]

        shouldNotEdges = [(0, 3)]

        self.generic_rag_test(rag=ragA,
                              numberOfNodes=4,
                              shouldEdges=shouldEdges,
                              shouldNotEdges=shouldNotEdges)

        self.generic_rag_test(rag=ragB,
                              numberOfNodes=4,
                              shouldEdges=shouldEdges,
                              shouldNotEdges=shouldNotEdges)

    @unittest.skipUnless(nifty.Configuration.WITH_HDF5, "skipping hdf5 tests")
    def test_hdf5_rag2d(self):
        import nifty.hdf5 as nhdf5

        shape = [3, 3]
        chunkShape = [1, 1]
        blockShape = [2, 2]

        hidT = nhdf5.createFile(self.path)
        array = nhdf5.Hdf5ArrayUInt32(hidT, "data", shape, chunkShape)

        self.assertEqual(array.shape[0], shape[0])
        self.assertEqual(array.shape[1], shape[1])

        labels = numpy.array([[0, 1, 1],
                              [2, 2, 2],
                              [3, 3, 3]], dtype='uint32')

        self.assertEqual(labels.shape[0], shape[0])
        self.assertEqual(labels.shape[1], shape[1])

        array[0:shape[0], 0:shape[1]] = labels

        rag = nrag.gridRagHdf5(array,
                               numberOfLabels=labels.max() + 1,
                               blockShape=blockShape,
                               numberOfThreads=2)

        shouldEdges = [(0, 1),
                       (0, 2),
                       (1, 2),
                       (2, 3)]

        shouldNotEdges = [(0, 3),
                          (1, 3)]

        self.generic_rag_test(rag=rag,
                              numberOfNodes=labels.max() + 1,
                              shouldEdges=shouldEdges,
                              shouldNotEdges=shouldNotEdges)
        nhdf5.closeFile(hidT)

    @unittest.skipUnless(nifty.Configuration.WITH_HDF5, "skipping hdf5 tests")
    def test_hdf5_rag2d_large(self):
        import nifty.hdf5 as nhdf5

        shape = [5, 6]
        blockShape = chunkShape = shape
        # FIXME  these shapes cause incorrect edges !!!
        # chunkShape = [3, 2]
        # blockShape = [2, 3]

        hidT = nhdf5.createFile(self.path)
        array = nhdf5.Hdf5ArrayUInt32(hidT, "data", shape, chunkShape)

        self.assertEqual(array.shape[0], shape[0])
        self.assertEqual(array.shape[1], shape[1])

        labels = numpy.array([[0, 0, 0, 0, 1, 1],
                              [0, 2, 2, 0, 1, 3],
                              [0, 3, 3, 3, 3, 3],
                              [0, 3, 4, 5, 5, 5],
                              [0, 0, 4, 6, 6, 6]],
                             dtype='uint32')

        self.assertEqual(labels.shape[0], shape[0])
        self.assertEqual(labels.shape[1], shape[1])

        array[0:shape[0], 0:shape[1]] = labels
        rag = nrag.gridRagHdf5(array,
                               numberOfLabels=int(labels.max() + 1),
                               blockShape=blockShape,
                               numberOfThreads=1)

        shouldEdges = [(0, 1),
                       (0, 2),
                       (0, 3),
                       (0, 4),
                       (1, 3),
                       (2, 3),
                       (3, 4),
                       (3, 5),
                       (4, 5),
                       (4, 6),
                       (5, 6)]

        shouldNotEdges = [(0, 6),
                          (0, 5),
                          (1, 6),
                          (1, 5)]

        self.generic_rag_test(rag=rag,
                              numberOfNodes=labels.max() + 1,
                              shouldEdges=shouldEdges,
                              shouldNotEdges=shouldNotEdges)
        nhdf5.closeFile(hidT)

    @unittest.skipUnless(nifty.Configuration.WITH_HDF5, "skipping hdf5 tests")
    def test_hdf5_rag_3d(self):
        import nifty.hdf5 as nhdf5

        shape = [3, 2, 2]
        chunkShape = [1, 2, 1]
        blockShape = [1, 2, 3]

        hidT = nhdf5.createFile(self.path)
        array = nhdf5.Hdf5ArrayUInt32(hidT, "data", shape, chunkShape)

        self.assertEqual(array.shape[0], shape[0])
        self.assertEqual(array.shape[1], shape[1])
        self.assertEqual(array.shape[2], shape[2])

        labels = [[[0, 1],
                   [0, 0]],
                  [[1, 1],
                   [2, 2]],
                  [[3, 3],
                   [3, 3]]]
        labels = numpy.array(labels, dtype='uint32')

        self.assertEqual(labels.shape[0], shape[0])
        self.assertEqual(labels.shape[1], shape[1])
        self.assertEqual(labels.shape[2], shape[2])

        array[0:shape[0], 0:shape[1], 0:shape[2]] = labels
        rag = nrag.gridRagHdf5(array,
                               numberOfLabels=labels.max() + 1,
                               blockShape=blockShape,
                               numberOfThreads=-1)

        shouldEdges = [(0, 1),
                       (0, 2),
                       (1, 2),
                       (1, 3),
                       (2, 3)]

        shouldNotEdges = [(0, 3)]

        self.generic_rag_test(rag=rag,
                              numberOfNodes=labels.max() + 1,
                              shouldEdges=shouldEdges,
                              shouldNotEdges=shouldNotEdges)
        nhdf5.closeFile(hidT)


if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import
from __future__ import print_function
from .import _cgp as __cgp
from ._cgp import *

from scipy.ndimage import grey_dilation
from skimage.transform import resize as __imresize
from scipy.ndimage import zoom as __zoom

try:
    import pylab
    import matplotlib.cm as cm
    __hasPyLabAndMatplotlib = True
except ImportError:
    __hasPyLabAndMatplotlib = False

__all__ = [
    'makeCellImage'
]

for key in __cgp.__dict__.keys():
    __all__.append(key)
    try:
         __cgp.__dict__[key].__module__='nifty.cgp'
    except:
        pass

import numpy

def _extenTGrid():


    def _gridView(self):
        a = self._gridView()
        a.flags.writeable = False
        return a

    TopologicalGrid2D.__array__ = _gridView

    def extractCellsBounds(self):
        return Bounds2D(self)

    def extractCellsGeometry(self, fill=True, sort1Cells=True):
        return Geometry2D(self, fill=fill,sort1Cells=sort1Cells)

    TopologicalGrid2D.extractCellsBounds = extractCellsBounds
    TopologicalGrid2D.extractCellsGeometry = extractCellsGeometry


    # filled tgrid

    FilledTopologicalGrid2D.__array__ = _gridView

    def cellMask2D(self, showCells):
        offset = self.cellTypeOffset
        a = numpy.array(self, copy=True)

        for cellType in [0,1,2]:
            if not showCells[cellType]:
                if cellType==0:
                    a[a>offset[0]]  = 0
                else:
                   a[ numpy.logical_and(a>offset[cellType], a<=offset[cellType-1])] = 0


        return a

    FilledTopologicalGrid2D.cellMask = cellMask2D

_extenTGrid()
del _extenTGrid



def __extend__():
    def getGeoItem(self, cellType):
        if cellType == 0:
            return self.cell0Geometry()
        elif cellType == 1:
            return  self.cell1Geometry()
        elif cellType == 2:
            return  self.cell2Geometry()
        else:
            return IndexError("cellType must be 0 1 or 2")

    Geometry2D.__getitem__ = getGeoItem


    def getBoundsItem(self, cellType):
        if cellType == 0:
            return self.cell0Bounds()
        elif cellType == 1:
            return  self.cell1Bounds()
        else:
            return IndexError("cellType must be 0 or 1")

    Bounds2D.__getitem__ = getBoundsItem


    # cell 1 bounded by (in 2D junctions of boundaries)
    def get_inverse1(self):
        return Cell1BoundedByVector2D(self)
    Cell0BoundsVector2D.reverseMapping = get_inverse1

    # cell 1 bounded by (in 2D junctions of boundaries)
    def get_inverse2(self):
        return Cell2BoundedByVector2D(self)
    Cell1BoundsVector2D.reverseMapping = get_inverse2


__extend__()
del __extend__



def makeCellImage(image, mask_image, lut, size=None):
    if image.shape[0:2] != mask_image.shape:
        image =   __imresize(image, mask_image.shape)#, interp='nearest')

    if size is not None:
        mask_image = grey_dilation(mask_image, size=size)
    if(not __hasPyLabAndMatplotlib):
        raise RuntimeError("showCellValues")
    else:
        if lut.ndim ==1:
            nLutChannels = 1
            zeroValue = 0
            _lut = numpy.hstack((zeroValue,lut))
            lutImg = numpy.take(_lut, mask_image)
            resImage = image.copy()
            whereImage = mask_image!=0
            resImage[whereImage] = lutImg[whereImage]
            return resImage


        elif lut.ndim ==2:
            lutC = lut.shape[1]
            if image.ndim == 2:
                image = numpy.concatenate([image[:,:,None]]*lutC,axis=2)

            nLutChannels = lut.shape[1]
            zeroValue = [0]*nLutChannels
            zeroValue = numpy.array(zeroValue)[None,:]
            _lut = numpy.concatenate((zeroValue,lut))

            #lutImg0
            #lutImg1
            #lutImg2




            lutImg = _lut[mask_image.ravel(),:].reshape(mask_image.shape+(lutC,))


            resImage = image.copy()
            whereImage = mask_image!=0
            resImage[whereImage] = lutImg[whereImage]
            return resImage


            #resImage = image.copy()
            #print("resImg",resImage.shape)
            #whereImage = mask_image!=0
            #for c in range(lutC):
            #    resImgC = resImage[:,:,c]
            #    lutImgC = lutImg[:,:,c]
            #    resImgC[whereImage] = lutImgC[whereImage]
            #return resImage

        else:
            raise ValueError("lut ndim must be in [1,2]")

        #print("theklut",_lut.shape,mask_image.shape)
        lutImg = numpy.take(_lut, mask_image)
        lutImg = _lut[mask_image.ravel(),:].reshape(mask_image.shape+(3,))
        #print("lut image",lutImg.shape)
        resImage = image.copy()
        whereImage = mask_image!=0
        resImage[whereImage] = lutImg[whereImage]
        return resImage






def cell1Features(tgrid, geometry=None, bounds=None,
        boundedBy=None,raw=None, pmap=None):


    if geometry is None:
        geometry = tgrid.extractCellsGeometry()
    if geometry is None:
        bounds = tgrid.extractCellsBounds()
    if boundedBy is None:
        boundedBy = {1:bounds[0].reverseMapping(),
                     2:bounds[1].reverseMapping()}


    feats = []
    names = []

    # curvature
    op = Cell1CurvatureFeatures2D()
    feat = op(cell1GeometryVector=geometry[1],
              cell1BoundedByVector=boundedBy[1])
    feats.append(feat)
    fNames = op.names()

    assert len(fNames) == feat.shape[1]

    names.extend(fNames)

    # line segment dist
    op = Cell1LineSegmentDist2D()
    feat = op(cell1GeometryVector=geometry[1])
    feats.append(feat)
    fNames = op.names()
    assert len(fNames) == feat.shape[1]
    names.extend(fNames)

    # basic geometric features
    op = Cell1BasicGeometricFeatures2D()
    feat = op(cell1GeometryVector=geometry[1],
             cell2GeometryVector=geometry[2],
             cell1BoundsVector=bounds[1])
    feats.append(feat)
    fNames = op.names()
    print(feat.shape, len(fNames))
    assert len(fNames) == feat.shape[1]
    names.extend(fNames)

    # basic topological features
    op = Cell1BasicTopologicalFeatures2D()
    feat = op(cell0BoundsVector=bounds[0],
              cell1BoundsVector=bounds[1],
              cell1BoundedByVector=boundedBy[1],
              cell2BoundedByVector=boundedBy[2])
    feats.append(feat)
    fNames = op.names()
    assert len(fNames) == feat.shape[1]
    names.extend(fNames)

    feats =  numpy.concatenate(feats, axis=1)
    assert len(names) == feats.shape[1]

    return feats, names


